proc astoControlPrefsAccept {} {
    global gvar

    set gvar(queueThreshold) $gvar(tmpQueueThreshold)
    set gvar(autoPilot)      $gvar(tmpAutoPilot)

    astoControlPrefsClose 
}

proc astoEditPrefs {} {

    panImportPanel astoPrefs

}

proc astoControlPrefsClose {} {
    global gvar

    panClosePanel no .astoPrefs
}


proc astoControlDefaults {} {
    global env 

    if {[file exists $env(ASTO_CONFIG_DIR)/astoControl.config] &&
	[file readable $env(ASTO_CONFIG_DIR)/astoControl.config]} {
	ScanConfigFile $env(ASTO_CONFIG_DIR)/astoControl.config defaults
    } else {
	Log "$env(ASTO_CONFIG_DIR)/astoControl.config does't exist. Exit" Error
	Info "$env(ASTO_CONFIG_DIR)/astoControl.config does't exist or is not readable.\nExit"
	exit 1
    }
    
    # List with all AstoControl Preferences Parameters.
    #This list at the same time
    # defines the default values for each paramater
    if {[catch { \
			set prefsList [list  \
			"cleanVolume    $defaults(cleanVolume)" \
			"fileCategory   $defaults(fileCategory)" \
			"host           \{$defaults(host)\}" \
			"parallelAsto   $defaults(parallelAsto)" \
		    "romanicAction  $defaults(romanicAction)" \
		    "dummyMode      $defaults(dummyMode)"] } msg ]} {
	ErrConf $msg
	exit 1
    }
		   
    astoControlSetDefaults $prefsList
}


proc writeControlDefaults {} {
    global gvar

    #if {[file exists $env(ASTO_CONFIG_DIR)/writeControl.config]} {
	#ScanConfigFile $env(ASTO_CONFIG_DIR)/writeControl.config defaults
    #} else {
#	Info "$env(ASTO_CONFIG_DIR)/writeControl.config does't exist. Exit"
#	exit 1
#    }
    
    # List with all AstoControl Preferences Parameters.
    #This list at the same time
    # defines the default values for each paramater
    set prefsList [list "romanicAction      $gvar(romanicAction)" \
	 	        "dummyMode          $gvar(dummyMode)"]
    writeControlSavePrefs  $prefsList
}

proc astoControlSetDefaults {prefsList} {
    global gvar defaults

    foreach el $prefsList {
	set key [lindex $el 0]	
	if {![info exists gvar($key)]} {
	    set value [lindex $el 1]
	    set gvar($key) $value
	    Debug "Set gvar($key) to $value" 3
	    set savePrefs 1	   
	}
    }
}

proc writeControlSavePrefs {prefsList} {
    global gvar env

    
    if {[catch {set file [open $env(ASTO_CONFIG_DIR)/writeControl.config w]} msg]} {
	Info "Cannot open $env(ASTO_CONFIG_DIR)/writeControl.config\n$msg"
	exit 1
    }

    #set gvar(autoPilot)      $gvar(tmpAutoPilot)
    #set gvar(queueThreshold) $gvar(tmpQueueThreshold)

    set fileBuf "\
	    ###########################################################\n\
	    #                                                         #\n\
	    #              THE WRITE CONTROL TOOL                     #\n\
	    #                                                         #\n\
	    #              writeControl Preferences                   #\n\
	    #                                                         #\n\
	    #              Don't edit this file manually.             #\n\
	    #                                                         #\n\
	    #              Edit it via Asto Control.                  #\n\
	    #                                                         #\n\
	    ###########################################################\n\n"
    set prefFormat "%-24s: %s\n"    
    foreach el $prefsList {
	set key [lindex $el 0]	
	set value $gvar($key)
	set line [format $prefFormat $key $value]
	set fileBuf "${fileBuf}$line"
    }
    set fileBuf "${fileBuf}\n\n\# --- oOo ---\n"    
    puts $file $fileBuf
    Debug "Buffer: $fileBuf" 3
    close $file
}
