# -*- Tcl -*-
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.IDENT     $Id: romanic.tcl,v 1.1 2004/09/06 14:47:14 cguirao Exp $
#.LANGUAGE  Tcl/Tk
#.AUTHOR    Fabio Sogni, ESO
#.PURPOSE   Server to control the RIMAGE CD Production Robot.
#.COMMENTS 
#   A deamon which accepts requests in form of ascii files
#   and control the production robot.
#
#.VERSION   1.0  01-Jul-00: FS, Creation
#--------------------------------------------------------------------

#Initialize the server, reset the robot and count the CDs
proc romanicProcInitialize {} {

    package require Tclx 

    Debug  "Define variables" 1
    romanicProcDefineVar
    
    Debug "Set options" 1
    romanicSetOptions
   
    Debug "Read configuration file" 1
    
    romanicReadConfFile 
   
    #puts "I'm the father, I go out"
    exit
}

#Second part of the initialization.
proc romanicProcInitializeLoop {} {
    global gvar

    set gvar(cancelGlobLoop) 0
    set gvar(cancelServeRequestsLoop) 0
    
    
    foreach tty $gvar(ttyList) {
		Debug "process [pid] takes care of tty: $tty" 3
	}
    
    foreach device [string trimright [string trimleft $gvar(deviceList) "{"] "}"] {
		Debug "process [pid] takes care of device: $device" 3
		Debug "which contains: $gvar($device)" 3
    }    
    
    Debug "Open serial ports" 1
    openSerialPorts   
    
    Debug  "Get sense" 1
    romanicRequestSenseAll

    Debug "Delete temporary files" 1
    cleanUpAll
    
    Debug "Reset robot" 1
    romanicResetRobotAll
    
    Debug "Calculate number of CDs" 1
    romanicCalculateNumCDsAll
    
    puts   "Robot $gvar(robotType) is ready to serve requests"
    Debug  "Robot $gvar(robotType) is ready to serve requests" 1
    romanicGlobLoop 3000 globLoop
    romanicServeRequestsLoop 3000 serveRequestsLoop
    mainLoop
}


#Define some constants and initialize the variables.
proc romanicProcDefineVar {} {
    global gvar devices
    
    set gvar(requests) ""
    set gvar(ttyList) "";    # list containing all the tty     
    set gvar(deviceList) ""; # list containing all the devices
    set gvar(speedList)  ""; # list containing the speed of the ttys 
    set gvar(badLabel) ""  ; # label to use in case the writing fails.

    if {[catch {exec which gs}]} {
	puts "Cannot find ghostscript (gs)"
	exit 1 
    }
#    if {[catch {exec which pfs_mount}]} {
#	puts "Cannot find pfs_mount"
#	exit 1 
#    }
    if {[catch {exec which cdrecord}]} {
	puts "Cannot find cdrecord"
	exit 1 
    }
}

#Procedure invoked in 3-magazine mode. 1^ magazine: input
#2^ magazine: output 3^ magazine: bad CDs
proc threeMagazineMode {request} {
    global gvar

    #Check if the request is valid.
    if {[checkRequest $request]} { exit 1 }
    set id [setId $request]
    set tty $gvar($gvar($id-writerAddress))
    set ttyTail [file tail $tty]
        
    set gvar(id)   $id
    set gvar(tty)  $tty
    set gvar(ttyTail) $ttyTail
    write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Processing:\n$gvar(id)"
    
    #Check that the arm is not moving, then get a blank.
    justWait $ttyTail
    lockArm $ttyTail
    getBlank 

    #Write a CD
    if {$gvar($id-write)} {
	putDiskInWriter $gvar($id-writerAddress)
	fanOn
	releaseArm $ttyTail
	write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Writing:\n$gvar(id)"
	set resultWrite [writeCD $id]
	justWait $ttyTail
	lockArm $ttyTail
	getDiskFromWriter $gvar($id-writerAddress)
    } else {
	set resultWrite "ok"
    }

    #If the writing failed put the CD in the "bad" magazine.
    if {$resultWrite == "error"} {
	putBad 
	writeError "Error writing media: $gvar($id-volumeFile)" 
    } else {
	#The writing went fine, print the CD (if requested)
	if {$gvar($id-print) } {
	Debug "printing..." 1
	    write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Printing:\n$gvar(id)"
	
	    putDiskInPrinter
	    
	    #puts "Printing..."
	    #after 3000
	    printLabel goodLabel
	    getDiskFromPrinter
	    
	}
	putWritten
	
	#I have to put here this call because LSK doesn't have a printer
	#and printLabel is not called.
	#if {![string match "VLT*" $gvar($id-volumeName)]} {
	#    printSummary $gvar($id-volumeName)
	#}
    }

    #enable the following line for testing
    #writeError "Error writing media: $gvar($id-volumeFile)" 

    file delete $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic 
    set inProcess [glob -nocomplain $gvar(reqDir)/fan-*-$gvar(ttyTail).romanic]
    if {[llength $inProcess] < 1} {
	fanOff
    }
    
    armOnTop 
    unLockDevice $gvar($id-writerAddress)
    releaseArm $ttyTail
    cleanUpQueue $id
}

#Procedure invoked in efficient mode. 1^ magazine: input
#2^ magazine: input 3^ magazine: output.
proc efficientMode {request} {
    global gvar 
    
    #Check if the request is valid.
    if {[checkRequest $request]} { exit 1 }
    set id  [setId $request]
    set tty $gvar($gvar($id-writerAddress))
    set ttyTail [file tail $tty]
    if {![file exists $gvar(reqDir)/request-$id.romanic]} {
	cleanUpQueue $id
    }

    set gvar(id)   $id
    set gvar(tty)  $tty
    set gvar(ttyTail) $ttyTail
    write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Processing:\n$gvar(id)"
    
    #Check that the arm is not moving, then get a blank.
    justWait $ttyTail
    lockArm $ttyTail
    getInputCd
    
    #Write a CD
    if {$gvar($id-write)} {
	putDiskInWriter $gvar($id-writerAddress)
	
	# I still have to decide wether to perform the check
	# of the blank or let cdrecord fail...(SOG)
	#if {[loadCD $gvar($id-writerAddress)] == "error"} {
	#    waitForFixError "I cannot read the media in $gvar($id-writerAddress)"
	#}
	fanOn
	releaseArm $ttyTail
	write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Writing:\n$gvar(id)"
	set resultWrite [writeCD $id]
	justWait $ttyTail
	lockArm $ttyTail
	getDiskFromWriter $gvar($id-writerAddress)
    } else {
	set resultWrite "ok"
    }
    

    #If requested print a label.
    if {($gvar($id-print) || $resultWrite == "error") } {
	write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Printing:\n$gvar(id)"
	putDiskInPrinter
	#If the writing went bad, print a "bad" label.
	if {$resultWrite == "error"} {
	    set label "badLabel"
	} else { set label "goodLabel" }
	printLabel   $label
	getDiskFromPrinter
	
	#Enable the following line for testing.
	#writeError "Error writing media: $gvar($id-volumeFile)" 
    }
    putOutputCd 
    
    if {$resultWrite == "error"} {
	writeError "Error writing media: $gvar($id-volumeFile)"
    } else {
	
	#I have to put here this call because LSK doesn't have a printer
	#and printLabel is not called.
	#if {![string match "VLT*" $gvar($id-volumeName)]} {
	#    printSummary $gvar($id-volumeName)
	#}
    }

    file delete $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic 
    set inProcess [glob -nocomplain $gvar(reqDir)/fan-*-$gvar(ttyTail).romanic]
    if {[llength $inProcess] < 1} {
	fanOff
    }
    armOnTop 
    unLockDevice $gvar($id-writerAddress)
    releaseArm $ttyTail
    cleanUpQueue $id
}

#Procedure invoked in mixed mode. 1^ magazine: CDs
#2^ magazine: DVDs 3^ magazine: output
proc mixedMode {request} {
    global gvar deviceType

    #Check if the request is valid.
    if {[checkRequest $request]} { exit 1 }
    set id [setId $request]
    set tty $gvar($gvar($id-writerAddress))
    set type $gvar($id-medium)
    set ttyTail [file tail $tty]

    set gvar(id)   $id
    set gvar(tty)  $tty
    set gvar(ttyTail) $ttyTail
    write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Processing:\n$gvar(id)"
    
    #Check that the arm is not moving, then get a blank.
    justWait $ttyTail
    lockArm $ttyTail
    
    if {$type == "CD"} {
	getBlank 
    } else {
	getWritten
    }
    
    #Write a Media
    if {$gvar($id-write)} {
	putDiskInWriter $gvar($id-writerAddress)
	#while {[loadCD $gvar($id-writerAddress)] == "error"} {
	#    ejectCD $gvar($id-writerAddress)
	#    waitForFixError "I cannot read the media in $gvar($id-writerAddress)\nRemember:\nMagazine 1 = CDs\nMagazine 2 = DVDs"
	#}
	
	fanOn
	releaseArm $ttyTail
	write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Writing:\n$gvar(id)"
	set resultWrite [writeCD $id]
	justWait $ttyTail
	lockArm $ttyTail
	getDiskFromWriter $gvar($id-writerAddress)
    } else {
	set resultWrite "ok"
    }
    
    #If the writing failed put the CD in the "bad" magazine.
    if {($gvar($id-print) || $resultWrite == "error") } {	
	write_file $gvar(reqDir)/inProcess-$id-$gvar(ttyTail).romanic "Printing:\n$gvar(id)"
	putDiskInPrinter
	after 3000
	#If the writing went bad, print a "bad" label.
	if {$resultWrite == "Error"} {
	    set label "badLabel"
	} else { set label "goodLabel" }
	printLabel   $label
	getDiskFromPrinter
	#Enable the following line for testing
	#writeError "Error writing media: $gvar($id-volumeFile)" 
    }
    putBad

    if {$resultWrite == "error"} {
	writeError "Error writing media: $gvar($id-volumeFile)"
    } else {
	#I have to put here this call because LSK doesn't have a printer
	#and printLabel is not called.
	#if {![string match "VLT*" $gvar($id-volumeName)]} {
	#    printSummary $gvar($id-volumeName)
	#}
    }

    file delete $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic 
    set inProcess [glob -nocomplain $gvar(reqDir)/fan-*-$gvar(ttyTail).romanic]
    if {[llength $inProcess] <1} {
	fanOff
    }
    
    armOnTop 
    unLockDevice $gvar($id-writerAddress)
    releaseArm $ttyTail
    cleanUpQueue $id
}



#Enable the event handler.
proc mainLoop {} {

    fileevent stdin readable fakeProc
    vwait forever    
}

#Trick to enable the event handler.
proc fakeProc {} {return}

#################### Main ###############

proc main {argv} {
    global gvar 

    set gvar(args) $argv
    romanicProcInitialize
}

main $argv


##########################################
