
proc romanicDefineCommands {} {
	global gvar
	
	switch $gvar(robotType) {
		"RIMAGE" {romanicDefineCommandsRimage}
		"LSK"    {romanicDefineCommandsLsk}
	}
}


#Open the printer drawer.
proc openPrinter {} {
    global gvar
    
    switch $gvar(robotType) {
	"RIMAGE" {openPrinterRimage}
	"LSK"    {openPrinterLsk}
    }
}

#Close the printer drawer.
proc closePrinter {} {
    global gvar 
	
	switch $gvar(robotType) {
		"RIMAGE" {closePrinterRimage}
		"LSK"    {closePrinterLsk}
	}
}


#Switch on the fan. There is only one fan for two writers.
proc fanOn {} {
    global gvar
    
	switch $gvar(robotType) {
		"RIMAGE" {fanOnRimage}
		"LSK"    {fanOnLsk}
	}
}

#Switch off the fan.
proc fanOff {} {
    global gvar
    
	switch $gvar(robotType) {
		"RIMAGE" {fanOffRimage}
		"LSK"    {fanOffLsk}
	}
	
}

#Send an enquiry message to the robot.
proc romanicPilotEnquiry {} {
    global gvar
	
	switch $gvar(robotType) {
		"RIMAGE" {romanicPilotEnquiryRimage}
		"LSK"    {romanicPilotEnquiryLsk}
	}
}
 

#Pick up a CD from a specified magazine and don't complain
#if there isn't any. It's an operation used to count the CDs.
proc pickUpCD {tty magazine} {
    global gvar

	switch $gvar(robotType) {
		"RIMAGE" {pickUpCDRimage $tty $magazine}
		"LSK"    {}
	}
}


proc dropCD {tty magazine} {
    global gvar 

	switch $gvar(robotType) {
		"RIMAGE" {dropCDRimage $tty $magazine}
		"LSK"    {}
	}
}


#Read the encoder steps of the arm. Used to count the CDs.
proc readElement {tty magazine} {
    global gvar

	switch $gvar(robotType) {
		"RIMAGE" {readElementRimage $tty $magazine}
		"LSK"    {}
	}
}



proc romanicRequestSense {} {
	global gvar
    
    switch $gvar(robotType) {
	"RIMAGE" { return [romanicRequestSenseRimage]}
	"LSK"    { return [romanicRequestSenseLsk]}
    }
}

#####  Move arm commands ###

proc getBlank {} {
    global gvar 
    
switch $gvar(robotType) {
		"RIMAGE" {getBlankRimage}
		"LSK"    {getBlankLsk}
	}
}

proc getWritten {} {
	global gvar
	
	switch $gvar(robotType) {
		"RIMAGE" {getWrittenRimage}
		"LSK"    {getWrittenLsk}
	}
}


proc getBad {} {
    global gvar 

	switch $gvar(robotType) {
		"RIMAGE" {getBadRimage}
		"LSK"    {getBadLsk}
	}
}



#Take a CD from the CD writer.
proc getMedia {} {
    global gvar 

	switch $gvar(robotType) {
		"RIMAGE" {getMediaRimage}
		"LSK"    {}
	}
}


proc getDiskFromWriter {writer} {
    global gvar
    
    switch $gvar(robotType) {
	"RIMAGE" {getDiskFromWriterRimage $writer}
	"LSK"    {getDiskFromWriterLsk    $writer}
    }
    
}



#Get a disk from a predefined "input magazine" (can be 1 or 2)
#It's used in the efficient mode, where the input can be magazine 1 or 2
proc getInputCd {} {
    global gvar

switch $gvar(robotType) {
		"RIMAGE" {getInputCdRimage}
		"LSK"    {getInputCdLsk}
	}

}


#Put a disk in the output magazine (can be 3 or 1). Used in efficient mode.
proc putOutputCd {} {
    global gvar
switch $gvar(robotType) {
		"RIMAGE" {putOutputCdRimage}
		"LSK"    {putOutputCdLsk}
	}
}



proc putDiskInWriter {writer} {
    global gvar
    
    switch $gvar(robotType) {
	"RIMAGE" {putDiskInWriterRimage $writer}
	"LSK"    {putDiskInWriterLsk    $writer}
    }
    
}


#Put a CD in a CD writer slot.
proc putMedia {} {
    global gvar 

	switch $gvar(robotType) {
		"RIMAGE" {putMediaRimage}
		"LSK"    {putMediaLsk}
	}

}


######

#Put a CD in the magazine 1.
proc putBlank {} {
    global gvar 
switch $gvar(robotType) {
		"RIMAGE" {putBlankRimage}
		"LSK"    {putBlankLsk}
	}

}


#Put a CD in magazine 2
proc putWritten {} {
    global gvar 

switch $gvar(robotType) {
		"RIMAGE" {putWrittenRimage}
		"LSK"    {putWrittenLsk}
	}

}


proc putBad {} {
    global gvar 

switch $gvar(robotType) {
		"RIMAGE" {putBadRimage}
		"LSK"    {putBadLsk}
	}

}

#Take a CD from the printer drawer.
proc getDiskFromPrinter {} {
    global gvar 

    switch $gvar(robotType) {
	"RIMAGE" {getDiskFromPrinterRimage}
	"LSK"    {getDiskFromPrinterLsk}
    }
}

#Put a CD on the printer drawer.
proc putDiskInPrinter {} {
    global gvar 

    switch $gvar(robotType) {
	"RIMAGE" {putDiskInPrinterRimage}
	"LSK"    {putDiskInPrinterLsk}
    }
}




#print a label.
proc printLabel {type} {
    global gvar 


    
    if {$type == "goodLabel"} { 
	set label [findVLTFile bitmaps/good-cd.ps]
    } else {
	set label [findVLTFile bitmaps/bad-cd.ps]
    }
    
    set psFile  [open $label r]
    set outFile [open "$gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).ps" w]
    set userName $gvar($gvar(id)-volumeName)
    set date [clock format [clock seconds] -format "%d-%b-%Y"]
    
    while {[gets $psFile line] > -1 } { 
        if {[string match "*CD-LABEL-HERE*" $line]} {
	    lvarpop line 3 ($userName)
	}
	
	if {[string match "*DD-MMM-YYYY*" $line]} {
	    lvarpop line 3 ($date)
	}
	
	puts $outFile $line
    }
    
    close $psFile
    close $outFile
    
    catch {exec ps2pcl $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).ps $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).rim}
    
    #Log "Printing... exec cat $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).rim > $gvar(tty)"
    #catch {exec cat $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).rim > $gvar(tty)}
    
    
    set file $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).rim
    switch $gvar(robotType) {
	"RIMAGE" {printLabelRimage $file}
	"LSK"    {printLabelLsk    $file}
    }
    
    file delete $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).rim
    file delete $gvar(reqDir)/cdLabel-$gvar($gvar(id)-volumeName).ps
    after 5000	
}


#Put the arm on top. It's kind of reset.
proc armOnTop {} {
    global gvar 

	switch $gvar(robotType) {
		"RIMAGE" {armOnTopRimage}
		"LSK"    {armOnTopLsk}
	}
}


#Move the arm to a specific position. 
proc positionMedium {address action} {
    global gvar

	switch $gvar(robotType) {
		"RIMAGE" {positionMediumRimage $address $action}
		"LSK"    {}
	}
}


proc checkWriters {tty} {
    global gvar 
    
    switch $gvar(robotType) {
	"RIMAGE" {checkWritersRimage $tty}
	"LSK"    {checkWritersLsk $tty}
    }
}

proc check {tty magazine} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { return [checkRimage $tty $magazine]}
	    "LSK"    { return [checkLsk $tty $magazine]}
	}
}

proc checkArm {tty} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { return [checkArmRimage $tty]}
	    "LSK"    { return [checkArmLsk $tty]}
	}
}

proc freeArm {tty} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { freeArmRimage $tty }
	    "LSK"    { checkArmLsk $tty }
	}
}

proc checkPrinter {tty} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { checkPrinterRimage $tty}
	    "LSK"    { checkPrinterLsk    $tty}
	}
}

proc checkMagazine {tty magazine} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { return [checkMagazineRimage $tty $magazine]}
	    "LSK"    { return [checkMagazineLsk    $tty $magazine]}
	}
}

proc calibrateArm {tty} {
    global gvar 
    
    switch $gvar(robotType) {
	"RIMAGE" { calibrateArmRimage $tty }
	"LSK"    { calibrateArmLsk    $tty }
    }
}


proc loopCommand {cmd attempts} {
    global gvar 

	switch $gvar(robotType) {
	    "RIMAGE" { loopCommandRobot $cmd $attempts}
	    "LSK"    { loopCommandRobot $cmd $attempts "X??"}
	}
}

#############################################