#!/bin/sh
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 1996 European Southern Observatory
# LICENSE
#
# PROJECT:   VLT Data Flow System
# AUTHOR:    Jeremy Maxfield
# CI NUMBER: 
# DESCRIPTION: Creates tar files containing deliverables for each machine
# REMARKS:     createDelivery  all | machine-name
#
# $Revision: 1.3 $
#******************************************************

#PATH=/sbin:/usr/sbin:/usr/bin
#export PATH
INSTALL_DIR=~flowmgr/dfsSrc
TAR_HOME=$PWD

ALL_MACHINES="primary \
              secondary "

#
# creates the tar file called dfs-$MACHINE.tar
#

function makeTarFile
{
   TAR_FILE=$DFS_VERSION-$MACHINE.tar.gz
   echo "$cmd: creating $TAR_FILE"
   (cd $INSTALL_DIR; tar cf  - $FILE_LIST | gzip >$TAR_HOME/$TAR_FILE)
   echo done
}

#
#
#  Adds a list of files according to the delivery list
#  according to the value of $PRODUCT
#  
#
function addToDelivery
{
   case $PRODUCT in

   'primary') FILE_LIST="\
	./$DFS_VERSION/INTROOT \
	./$DFS_VERSION/admin \
	./$DFS_VERSION/bin \
	./$DFS_VERSION/help \
	./$DFS_VERSION/man \
	./$DFS_VERSION/data \
	./$DFS_VERSION/log/logSql \
	./$DFS_VERSION/boot \
	./$DFS_VERSION/dataflowJava \
	./$DFS_VERSION/cpl \
	./$DFS_VERSION/sql "
	;;

   'secondary') FILE_LIST="\
	./$DFS_VERSION/admin/installBoot.sh \
	./$DFS_VERSION/bin/dfs-launch \
	./$DFS_VERSION/bin/show-olas \
	./$DFS_VERSION/bin/dbrcGet \
	./$DFS_VERSION/bin/olas-startup \
	./$DFS_VERSION/bin/cleanup-vcsolac \
	./$DFS_VERSION/bin/show-vcsolac \
	./$DFS_VERSION/bin/start-vcsolac \
	./$DFS_VERSION/bin/vcsolac \
	./$DFS_VERSION/bin/fitsTranslate \
	./$DFS_VERSION/bin/fitsVerify \
	./$DFS_VERSION/bin/vcsolacControl \
	./$DFS_VERSION/bin/sendFiles \
	./$DFS_VERSION/boot/vcsolac \
	./$DFS_VERSION/data/SYSTEM/fvDictionary/FV-DIC.TEST \
	./$DFS_VERSION/man/man1/vcsolac.1 \
	./$DFS_VERSION/man/man1/vcsolacControl.1 \
	./$DFS_VERSION/man/man1/fitsVerify.1 \
	./$DFS_VERSION/admin/README_vcsolac.doc "
    	;;

   *)	echo "$cmd: unknown product $PRODUCT"
   	echo "Usage: $cmd [release] [primary|secondary|all]"
	return 1
	;;

   esac
   return 0
}

#
#
# Makes a tar file for the $MACHINE
#
#
#
function makeTarForMachine
{
   case $MACHINE in

   'primary')  PRODUCT=primary
               addToDelivery  
               ;;

   'secondary')PRODUCT=secondary
               addToDelivery
               ;;
   *)          echo "$cmd: unknown delivery $MACHINE"
   	       echo "Usage: $cmd [release] [primary|secondary|all]"
               return 1
               ;;

   esac
   makeTarFile
   return 0
}

cmd=`basename $0` 
if [ -z "$2" -o -z "$1" ] ;then
   echo "Usage: $cmd [release] [primary|secondary|all]"
   exit 1
fi

DFS_VERSION=$1
cd $INSTALL_DIR
if [ ! -d "$1" ]; then
   echo "$cmd: directory $DFS_VERSION does not exist in $INSTALL_DIR" 
   echo "Usage: $cmd [release] [primary|secondary|all]"
   exit 1
fi

case $2 in
   'all')   
               for MACHINE in $ALL_MACHINES
               do
                  FILE_LIST=""
                  makeTarForMachine
               done
               ;;

    *)         MACHINE=$2
               FILE_LIST=""
               makeTarForMachine
               ;;

esac
exit 0
