package org.eso.dfs.services.data;

import java.io.File;

import org.eso.dfs.services.ServiceException;

/**
 * @author Nick Kornweibel
 *
 * This class is the Handler participant in the AssociationService.
 * The AssociationDisplay service follows the chain of responsibility pattern,
 * modified in that all handlers are used during processing, rather than
 * stopping after the first handler reports that it has processed the nodes.
 * (is there a name for this pattern?).
 */

public interface AssociationHandler {
    
    /**
     * Process the requested node array. All handlers in the chain are
     * will be passed the nodes and may modify the nodes in the array
     * as required. For example one handler may only perform the classification,
     * another the grouping, and so forth.
     * 
     * On error a ServiceException should be thrown. This informs the 
     * AssociationService that an error has occurred, but does not
     * (necessarily) stop the processNodes() call on remaining handlers.
     */
    public void processNodes(AssociationNode[] nodes) throws ServiceException;
    
    /**
     * Not sure if this method belongs here, but it certainly belongs under
     * the "data services" interface somewhere.
     * 
     * Add files to the data model, throw an exception if the files can not
     * be read or if there is an error handling the files.
     * 
     * @param f
     * @throws ServiceException
     */
    public void addFiles(File[] files) throws ServiceException;
    
    /**
     * Not sure if this method belongs here, but it certainly belongs under
     * the "data services" interface somewhere.
     * 
     * Remove listed files from the data model.
     * 
     * @param f
     * @throws ServiceException
     */
    public void removeFiles(File[] files);
    
}
