/**
 * @author Michele Zamparelli
 * @version 
 * heavily copied from the TreeControlPanel
 * A JPanel for controlling a DBTreeTable
 * 
 * TODO: refactoring: remove dependency on GasganoDisplay.
 * 
 */

package org.eso.gasgano;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import org.eso.gasgano.gui.DBTreeTableModel;
import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.gui.JTreeTable;
import org.eso.gasgano.gui.TreeModelIterator;
import org.eso.gasgano.properties.GasProp;
import org.eso.gasgano.properties.PropertyDB;

public class DBTreeControlPanel
    extends JPanel
    implements ActionListener, TreeSelectionListener {

    private JComboBox groupSelector = null;
    private JButton expandTree = null;
    private JButton collapseTree = null;
    private JTextField findEntry = null;
    private JButton findButton = null;

    private JTreeTable treeTable = null;
    private DBTreeTableModel model = null;

    private Object endSearchNode = null;

    private static final String groupByProgLabel = "Default grouping";
    private static final String groupByTelLabel = "Group by Telescope";
    private static final String findLabel = "find";

    private JLabel selectedNumber = null;
    private JComboBox periodSelector = null;
    private JComboBox instrumentSelector = null;
    private JComboBox searchType = null;

    // just keeps track of initial clicking of periodSelector JComboBox
    private boolean userPeriodSelection = false;
    private boolean userInstrumentSelection = false;

    private GasganoDisplay mainDisplay = null;

    public DBTreeControlPanel(
        JTreeTable t,
        DBTreeTableModel m,
        GasganoDisplay gd) {
        treeTable = t;
        model = m;
        mainDisplay = gd;
        init();
    }

    private void init() {
        Vector items = new Vector(2);
        setLayout(new BorderLayout());
        items.addElement(groupByProgLabel);
        items.addElement(groupByTelLabel);
        groupSelector = new JComboBox(items);
        groupSelector.addActionListener(this);

        items = new Vector(GasProp.periodMax - GasProp.periodMin + 1);

        items.addElement("All");
        for (int z = GasProp.periodMin; z <= GasProp.periodMax; z++) {
            items.addElement(String.valueOf(z));
        }

        periodSelector = new JComboBox(items);
        periodSelector.addActionListener(this);
        periodSelector.setSelectedItem(
            (String) PropertyDB.getInstance().getProperty("DBOBS_PERIOD"));

        items = new Vector(GasProp.getProperties().dbInstruments.size() + 1);

        items.addElement("All");
        for (int z = 0;
            z < GasProp.getProperties().dbInstruments.size();
            z++) {
            items.addElement(
                GasProp.getProperties().dbInstruments.elementAt(z));
        }

        instrumentSelector = new JComboBox(items);
        instrumentSelector.addActionListener(this);
        if (GasProp
            .getProperties()
            .dbInstruments
            .contains(
                PropertyDB.getInstance().getProperty("DBOBS_INSTRUMENT"))) {
            instrumentSelector.setSelectedItem(
                (String) PropertyDB.getInstance().getProperty(
                    "DBOBS_INSTRUMENT"));
        }

        items = new Vector(2);
        items.addElement("Find entry:");
        items.addElement("Add program:");

        searchType = new JComboBox(items);

        JLabel serverIdLabel =
            new JLabel(PropertyDB.getInstance().getProperty("DBOBS_SERVER_ID"));
        serverIdLabel.setFont(new Font("Dialog", Font.PLAIN, 18));
        serverIdLabel.setForeground(GUIUtils.dataBaseColor);
        serverIdLabel.setBorder(new EmptyBorder(3, 14, 3, 6));

        JPanel searchPanel = new JPanel();
        //searchPanel.add(new JLabel("Find entry:"));
        findEntry = new JTextField(20);
        findEntry.addActionListener(this);
        searchPanel.add(searchType);
        searchPanel.add(findEntry);

        //findButton = new JButton(findLabel);
        //findButton.addActionListener(this);
        //searchPanel.add(findButton);
        searchPanel.setBorder(new LineBorder(Color.black, 1));
        JPanel centerPanel = new JPanel();

        centerPanel.add(groupSelector);
        centerPanel.add(new JLabel("Instrument "));
        centerPanel.add(instrumentSelector);
        centerPanel.add(new JLabel("Period "));
        centerPanel.add(periodSelector);
        centerPanel.add(searchPanel);

        add(centerPanel, "Center");

        JPanel eastPanel = new JPanel(new BorderLayout());
        selectedNumber = new JLabel("0");
        selectedNumber.setBorder(new EmptyBorder(3, 1, 10, 6));
        treeTable.getTree().addTreeSelectionListener(this);
        eastPanel.add(selectedNumber, "West");

        ToolTipManager.sharedInstance().setInitialDelay(50);

        add(serverIdLabel, "West");
        add(eastPanel, "East");
    }

    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == groupSelector) {
            String item = groupSelector.getSelectedItem().toString();
            if (item == groupByProgLabel) {
                groupBy(GasProp.TREE_DEFAULT);
            }
            if (item == groupByTelLabel) {
                groupBy(GasProp.TREE_TELESCOPE);
            }
        }
        if (a.getSource() == findEntry) {
            if (searchType.getSelectedIndex() == 0) {
                if (searchTree(findEntry.getText()) == false) {
                    userMessage("\'" + findEntry.getText() + "\' not located.");
                }
            }
            if (searchType.getSelectedIndex() == 1) {
                model.addSingleProgram(findEntry.getText());
                findEntry.setText("");
            }

        }
        if (a.getSource() == periodSelector) {
            if (userPeriodSelection) {
                new Thread(new Runnable() {
                    public void run() {
                        String item =
                            periodSelector.getSelectedItem().toString();
                        GasProp.getProperties().put("DBOBS_PERIOD", item);
                        mainDisplay.setDatabasePanelEnabled(false);
                        if (model.periodSelectionChanged()) {
                            mainDisplay.setDatabasePanelEnabled(true);
                        }
                        // problem to be solved here
                        model.updateTree();
                        GasProp.getProperties().saveProperties();
                    }
                }).start();
            }
            userPeriodSelection = true;
        }
        if (a.getSource() == instrumentSelector) {
            if (userInstrumentSelection) {
                new Thread(new Runnable() {
                    public void run() {
                        String item =
                            instrumentSelector.getSelectedItem().toString();
                        GasProp.getProperties().put(
                            "DBOBS_INSTRUMENT",
                            item);
                        mainDisplay.setDatabasePanelEnabled(false);
                        if (model.periodSelectionChanged()) {
                            mainDisplay.setDatabasePanelEnabled(true);
                        }
                        // problem to be solved here
                        model.updateTree();
                        GasProp.getProperties().saveProperties();
                    }
                }).start();
            }
            userInstrumentSelection = true;
        }

    }

    public boolean searchTree(String findString) {
        boolean rc = false;
        TreePath selPath = treeTable.getTree().getLeadSelectionPath();
        boolean found = false;
        TreeModelIterator iterator = null;
        Object currentNode;
        int columnCount = model.getColumnCount();

        findString = findString.toUpperCase();
        if (selPath != null) {
            endSearchNode = selPath.getLastPathComponent();
        } else {
            endSearchNode = model.getRoot();
        }
        iterator = new TreeModelIterator(model, selPath);

        do {
            iterator.iteratePath();
            currentNode = iterator.getCurrentPath().lastElement();
            for (int i = 0; i < columnCount && !found; i++) {
                Object cellValue = model.getValueAt(currentNode, i);
                if (cellValue != null) {
                    String nodeVal = cellValue.toString().toUpperCase();
                    if (nodeVal.indexOf(findString) != -1) {
                        found = true;
                    }
                }
            }
        } while (currentNode != endSearchNode && !found);
        if (found) {
            Vector pathVector = iterator.getCurrentPath();
            Object[] path = new Object[pathVector.size()];
            int i = 0;
            while (i < path.length) {
                path[i] = pathVector.elementAt(i);
                i++;
            }
            TreePath tp = new TreePath(path);
            treeTable.getTree().setSelectionPath(tp);
            // does not work for JTreeTable.
            // treeTable.getTree().scrollPathToVisible(tp);
            // do as per a normal JTable instead:
            Rectangle r =
                treeTable.getCellRect(treeTable.getSelectedRow(), 0, true);
            treeTable.scrollRectToVisible(r);

        }
        return found;
    }

    public void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }

    public void groupBy(int gbLevel) {
        if (model instanceof DBTreeTableModel) {
            ((DBTreeTableModel) model).setGroupDataByLevel(gbLevel);
        }
    }

    public Component getFindEntryComponent() {
        return findEntry;
    }


    /**
     * @param command The command and arguments to be executed.
     * @return The return code of the completed execution.
     */
    private int runCommand(String command) {
        int rc = -1;
        try {
            Runtime r = Runtime.getRuntime();
            Process p = r.exec(command);
            try {
                p.waitFor();
                rc = p.exitValue();
            } catch (InterruptedException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            } catch (IllegalThreadStateException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            }
        } catch (IOException e) {
            System.out.println("Error executing command: " + e.toString());
        }
        return rc;
    }

    // method for the TreeSelectionListener()
    public void valueChanged(TreeSelectionEvent e) {
        Vector files = ((DBTreeTable) treeTable).getSelectedOBComponents();
        if (files == null) {
            selectedNumber.setText("0");
        } else {
            selectedNumber.setText(String.valueOf(files.size()));
        }
    }

}
// end of file TreeControlPanel.java
