/**
 * @author N. Kornweibel
 * @version 1.0
 * A class for managing and notifying DataModelChangeListener objects
 */

package org.eso.gasgano.datamodel;

import java.util.Enumeration;
import java.util.Vector;

import org.eso.gasgano.properties.PropertyDB;

public class DataModelChangeNotifier {

    private Vector dataModelListeners = null;

    public DataModelChangeNotifier() { }

    /**
     * @param l The DataModelChangeListener object to be added.
     */
    public void addDataModelChangeListener(DataModelChangeListener l) {
       if (dataModelListeners == null) {
            dataModelListeners = new Vector();
        }
       dataModelListeners.addElement(l);
    }
 
    /**
     * @param l The DataModelChangeListener object to be removed.
     */
    public void removeDataModelChangeListener(DataModelChangeListener l) {
        if (dataModelListeners != null) {
            dataModelListeners.removeElement(l);
        }
    }
 
    /**
     * @param evt The event to be communicated to all registered listeners.
     */
    public void fireDataModelChangeEvent(DataModelEvent evt) {
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugEvent )) {
            System.out.print("fireDataModelChangeEvent:");
            if (dataModelListeners == null) {
                System.out.println(" for 0 listeners.");
            } else {
                System.out.println(" for "+dataModelListeners.size()
                   +" listeners.");
            }
        }
        if (dataModelListeners != null) {
            // make a copy of the array, incase it is modified
            // during looping.
            Vector listenerCopy = (Vector)dataModelListeners.clone();
            Enumeration en = listenerCopy.elements();
            while (en.hasMoreElements()) {
                DataModelChangeListener l =
                    (DataModelChangeListener)en.nextElement();
                if (l != null) {
                    l.dataModelChanged(evt);
                }
            }
        }
    }
}
// end of file DataModelChangeNotifier.java.
