package org.eso.gasgano.datamodel.filesystem;

import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.KeywordBlock;

/**
 * @author N. Kornweibel for ESO.
 *
 * This class represents a KeywordBlock that is a FitsFile extension.
 * It is simply a KeywordBlock with a name, which stored in the
 * Keyword.EXTENSION_METAKEYWORD keyword.
 */

public class FitsExtKeywordBlock extends KeywordBlock {
    
    
    /**
     * Constructor - adds the Keyword.EXTENSION_METAKEYWORD keyword as the
     * first keyword (as a place holder) although the value is not set.
     */
    public FitsExtKeywordBlock() {
        addKeyword(new Keyword(Keyword.EXTENSION_METAKEYWORD, ""));
    }
    
    /**
     * get the name of this extension (stored in the Keyword.EXTENSION_METAKEYWORD)
     * keyword, or "" if not present.
     * @return String
     */
    public String getName() {
        String rc = "";
        Keyword k = getKeyword(Keyword.EXTENSION_METAKEYWORD);
        if (k != null) {
            rc = k.getValue();
        }
        return rc;
    }
    
    /**
     * set the name of this extension in the Keyword.EXTENSION_METAKEYWORD 
     * keyword if not null.
     * @param s
     */
    public void setName(String s) {
        if (s != null) {
            replaceKeyword(new Keyword(Keyword.EXTENSION_METAKEYWORD, s));
        }
    }
    
    /**
     * A utility method to generate the name of an extension based on keywords
     * in the keyword block and the extension number (starting at 0, although
     * the value included in the name starts at 1, i.e. extNum+1). Note extNum
     * is not currently used.
     * This name is:
     * <EXTENSION>,<EXTNAME>,<EXTVERSION>
     * Missing keywords are ignored in the creation of the name.
     * @param kb the keyword block to query data from
     * @param extNum the number of the extension (used in the name).
     * @return String the name of the extension.
     */
    public static String generateExtensionName(KeywordBlock kb, int extNum) {
        StringBuffer rc = new StringBuffer();
        
        Keyword ext = kb.getKeyword(Keyword.EXTENDSTART);
        Keyword extName = kb.getKeyword(Keyword.EXTENSION_NAME);
        Keyword extVer = kb.getKeyword(Keyword.EXTENSION_VERSION);
        
        if (ext != null) {
            rc.append(ext.getValue());
        }
        if (extName != null) {
            if (rc.length() > 0) {
                rc.append(", ");
            }
            rc.append(extName.getValue());
        }
        if (extVer != null) {
            if (rc.length() > 0) {
                rc.append(", ");
            }
            rc.append(extVer.getValue());
        }       
        //return ""+(extNum+1)+" "+rc.toString();
        return rc.toString();
    }
}
