/**
 * @author N. Kornweibel
 * @version 1.0
 * A dialog for executing the update() method of the DataModel in a
 * seperate thread.
 */

package org.eso.gasgano.gui;

import java.awt.Frame;
import java.util.Vector;

import org.eso.gasgano.datamodel.filesystem.DFSDataModel;

public class DataModelBuilder extends TaskDialog {

    private DFSDataModel dataModel = DFSDataModel.getDataModel();
    private Vector vectorOfFileVectors = null;
    private int filesToDo = 0;
    private int filesDone = 0;
    private static boolean allDone = false;

    public DataModelBuilder(DFSDataModel dm, Frame parent,
            String title, boolean modal) {
        super(parent, title, modal);
        dataModel = dm;
      if (dm == null) { 
	System.out.println("Data Model is NULL");
	System.exit(0);
      }
    }

    public void initTask() {
        filesToDo = 0;
        filesDone = 0;
        allDone = false;
        vectorOfFileVectors = dataModel.getFileSet();
        super.initTask();
    }

    /**  
     * This method must launch a thread to execute the task required.
     * see header for a typical example.
     */
    public void go() {
        final SwingWorker worker = new SwingWorker() {
            public Object construct() {
                return new BuilderThread();
            }
        };
    }

    /**
     * Called from Timer to find out how much work needs
     * to be done.
     */
    public int getLengthOfTask() {
        filesToDo = dataModel.getTotalFilesInSet();
        return filesToDo;
    }

    /**
     * Called from Timer to find out how much has been done.
     */
    public int getCurrent() {
        filesDone = dataModel.getNumberOfFilesUpdated();
        return filesDone;
    }

  protected static void setDone(boolean george) { 
    allDone = george;
  }

    /**
     * Called from Timer to find the progress status.
     */
    public String getProgressMessage() {
        return new String("Read "+filesDone+" of "
            +filesToDo+" files.");
    }

    /**
     * Called from Timer to find out if the task has completed.
     */
    public boolean done() {
        return allDone;
    }

    /**
     * The actual task.  This runs in a SwingWorker thread.
     */
    class BuilderThread {
        BuilderThread () {
	  DFSDataModel.getDataModel().update(DFSDataModel.getDataModel().getFileSet() ) ;
	  // vectorOfFileVectors);
	  //      allDone = true;
	  DataModelBuilder.setDone(true);
        }
    }
}
