package org.eso.gasgano.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;

/**
 * A JButton that binds a JFileChooser with a text field. When the button is pressed
 * the text field is read and set as the default path for the file chooser. When the
 * file is selected by the file chooser, the text field is updated with the selection.
 * 
 * The class only supports single file selection for now - that is, only a single file 
 * selection path is writen back to the text field.
 * 
 * @author Nick Kornweibel.
 */
public class FileChooserJButton extends JButton {

    private final JTextField textField;
    private final JFileChooser fileChooser;
    private String fileChooserTitle = "";
    private String fileChooserApproveButtonText = "Ok";

    /**
     * Create the FileChooserJButton connecting the given
     * text field and file chooser.
     * @param tField
     * @param fChooser
     */
    public FileChooserJButton(JTextField tField, JFileChooser fChooser) {
        textField = tField;
        fileChooser = fChooser;
        setText("...");
        // set up the actions
        this.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String path = null;
                if (textField != null && fileChooser != null) {
                    fileChooser.setDialogTitle(fileChooserTitle);
                    path = textField.getText();
                    if (path.length() > 0) {
                        fileChooser.setSelectedFile(new File(path));
                    }
                    if (fileChooser
                        .showDialog(
                            FileChooserJButton.this,
                            fileChooserApproveButtonText)
                        == JFileChooser.APPROVE_OPTION) {
                        File f = fileChooser.getSelectedFile();
                        textField.setText(f.getPath());
                    }

                }
            }
        });
    }

    /**
     * Returns the fileChooser.
     * @return JFileChooser
     */
    public JFileChooser getFileChooser() {
        return fileChooser;
    }

    /**
     * Returns the fileChooserTitle.
     * @return String
     */
    public String getFileChooserTitle() {
        return fileChooserTitle;
    }

    /**
     * Returns the textField.
     * @return JTextField
     */
    public JTextField getTextField() {
        return textField;
    }

    /**
     * Sets the fileChooserTitle.
     * @param fileChooserTitle The fileChooserTitle to set
     */
    public void setFileChooserTitle(String fileChooserTitle) {
        this.fileChooserTitle = fileChooserTitle;
    }

    /**
     * Returns the fileChooserApproveButtonText.
     * @return String
     */
    public String getFileChooserApproveButtonText() {
        return fileChooserApproveButtonText;
    }

    /**
     * Sets the fileChooserApproveButtonText.
     * @param fileChooserApproveButtonText The fileChooserApproveButtonText to set
     */
    public void setFileChooserApproveButtonText(String fileChooserApproveButtonText) {
        this.fileChooserApproveButtonText = fileChooserApproveButtonText;
    }

}
