/**
 * @author N. Kornweibel
 * @version 1.0
 * 
 * A simple dialog which will execute a ReportRunner object passed and wait on
 * its termination. If the "Cancel" button is pressed, the thread is
 * terminated.
 * 
 * The Dialog will dispose() of its self if Cancel is pressed or the
 * Thread terminates.
 */
package org.eso.gasgano.gui;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.Timer;

public class ReportDialog extends JDialog implements ActionListener {

    private JLabel userLabel;
    private String userMessage;
    private JButton cancelButton;
    private boolean interrupted = false;
    private Thread theThread;
    private ReportRunner runner;
    private Timer threadCheckTimer;

    public ReportDialog(Frame p, String title, String msg,
            boolean modal, ReportRunner r) {
        super(p, title, modal);
        userMessage = msg;
        theThread = new Thread(r);
        runner = r;
    }

    /**
     * Start the report thread and check timer.
     */
    public void run() {
        setupDialog();
        pack();
        setLocation(400, 400);
        theThread.start();
        threadCheckTimer = new Timer(1000, this);
        threadCheckTimer.start();
        show();
    }

    public boolean cancelSelected() {
        return interrupted;
    }

    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == threadCheckTimer) {
            // stop the timer to process the event.
            threadCheckTimer.stop();
            try {
                theThread.join(1000);
            } catch (InterruptedException ie) {
                endDialog();
            }
            if (theThread.isAlive() == false) {
                endDialog();
            } else {
                // thread still running - restart the timer.
                threadCheckTimer.restart();
            }
        } else if (a.getSource() == cancelButton) {
            try {
                runner.interruptReport();
            } catch (SecurityException e) {
                System.out.println("Security Exception stopping thread.");
            }
            interrupted = true;
        }
    }

    private void endDialog() {
        setVisible(false);
        dispose();
    }

    private void setupDialog() {
        userLabel = new JLabel(userMessage);
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        getContentPane().add(userLabel, "Center");
        JLabel l = new JLabel(new ImageIcon(
			ReportDialog.class.getResource(GUIUtils.hrglasGif)));
        getContentPane().add(l, "West");
        getContentPane().add(cancelButton, "South");
    }
}
// end of file ReportDialog.java.
