package org.eso.ohs.core.dbb.client;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.utilities.MsgManager;
import org.jdom.Element;


public class Utilities {
	
	/**
     * Field stdlog_
     */
    private static Logger stdlog_ = Logger.getLogger(Utilities.class);
	/** @return true if input object is a DbbPanel, false otherwise */
	
	private static HashMap queriesToBeTested = new HashMap();
	
	public static boolean isDbbPanel( Object obj ) {
		/*
		 * 	[Design by contract preconditions]
		 *  assert (obj != null): new IllegalArgumentException(MsgManager.errContract("null argument"));
		 */
		if(obj == null) throw new IllegalArgumentException(MsgManager.errContract("null argument"));
		/* DbC */
		return isSameClass(obj.getClass(), DbbPanel.class);
	}
	  
    
	/** @return true if input object is a DbbWidget, false otherwise */
	public static boolean isDbbWidget(Object obj) {
		/*
		 * 	[Design by contract preconditions]
		 *  assert (obj != null): new IllegalArgumentException(MsgManager.errContract("null argument"));
		 */
		if(obj == null) throw new IllegalArgumentException(MsgManager.errContract("null argument"));
		/* DbC */
		return isSameClass(obj.getClass(), DbbWidget.class);
	}
	/**
	 * Return true if and only if the arguments are the same class.
	 * @param a A class.
	 * @param b A class
	 * @return a boolean value
	 */
	private static boolean isSameClass(Class a, Class b){
		/* Design by Contract */
		assert (a != null && b!=null): new IllegalArgumentException(MsgManager.errContract("One or both classes are null"));
		/* DbC */
		return  (a.isAssignableFrom(b) && b.isAssignableFrom(a));
	}
	
	/** @return true if input object is a DbbWidget or a specialisation of it. False otherwise */
	public static boolean isDbbWidgetChild( Object obj ) {
		/* 	Design by Contract */
		if(obj == null) throw new IllegalArgumentException(MsgManager.errContract("null argument"));
		/* DbC */
		boolean b;
		if(obj == null) b = false;
		else {
		  Class   c = obj.getClass();
		  Class   w = DbbWidget.class;
		  b = w.isAssignableFrom( c );
		}
		return b;
	}
	
	/**
	 * It filters out all non-Element children of an Element.
	 * 
	 * @param element A JDOM Element.
	 * @return An iterator over all child Elements of the
	 *         input Element.
	 */
	public static Iterator getChildElements(Element element) {
		/* 	Design by Contract */
		if(element == null) throw new IllegalArgumentException(MsgManager.errContract("null argument"));
		/* DbC */
		Vector   children = new Vector();
		List      content = element.getContent();
		Iterator iterator = content.iterator();
        
		while( iterator.hasNext() ) {
			Object o = iterator.next();
			if( o instanceof Element ) {
				Element child = (Element) o;
				children.add( child );
			}
		}
		return children.iterator();
	}
	
	
	
	
	public static void testFor(String tableName){
		String query = new String();
		query = "select * from " + tableName;
		if (queriesToBeTested.get(query) == null)
			queriesToBeTested.put(query, query);
	}
	
	public static void testFor(String tableName, String column){
		String query = new String();
		query = "select " + column + " from " + tableName;
		if (queriesToBeTested.get(query) == null)
			queriesToBeTested.put(query, query);
	}
	
	/**
	 * This method query the database to check that the XML file 
	 * doesn't contains any wrong or mispelled table/column name.
	 * @param engine A DbbSqlEngineImpl used to access the database
	 * @return True if all the tables and columns exist in the db specified by the XML file.
	 */
	public static boolean checkForExistence(DbbSqlEngineImpl engine){
		boolean result = false;
		String query = "";
		try {
			stdlog_.debug(	"\n*************************\n" +
							"  STARTING SANITY CHECK\n" + 
							"*************************\n" +
							"(query the database to check that the XML file doesn't\n" +
							" contains any wrong or mispelled table/column name for\n" +
							"tables and columns name)\n\n");
			engine.setNoExec("on");
			Iterator i = queriesToBeTested.values().iterator();
			while (i.hasNext()){
				query = (String) i.next();
				stdlog_.debug("[probing] " + query);
				engine.execSqlNoResulset(query);
			}
			engine.setNoExec("off");
		} catch (SQLException e) {
			stdlog_.error("QUERY FAILED: " + query);
			result = false;
			e.printStackTrace();
			System.exit(-1);
		}
		stdlog_.debug(	"\n*************************\n" +
						" SANITY CHECK SUCCESFULL\n" + 
						"*************************\n");
		return result;
	}
	
}