package org.eso.ohs.core.dbb.client;

import java.util.HashMap;

import org.eso.ohs.core.dbb.xml.DbbXMLException;
import org.eso.ohs.core.utilities.MsgManager;


public class WidgetRegistry extends Registry{
	
	public WidgetRegistry(){
		registry = new HashMap();
	}
	
	/**
	 * Retrieve a DBB element by id.
	 * 
	 * @param id   Key to retrieve the widget. If the input XML document
	 *             contains for instance an element like:
	 *             <pre>
	 *               &lt;<DbbSqlChunkGroupMember id="rated_chunk47" column="grade".../> /&gt;
	 *             </pre>
	 *             this method allows to retrieve the corresponding
	 *             <code>DbbSqlChunkGroupMember</code> using key <em>rated_chunk47</em>.
	 * 
	 * @return The widget registered with the input id, if any;
	 *         <tt>null</tt> otherwise.
	 */
	public DbbWidget getWidget(String id ){
		 /* Design by Contract */
		 if ((id == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("id attribute is null"));
		 /* DbC */
		return (DbbWidget) registry.get(id);
	}
	/**
	 * Insert a new DbbWidget in the widget registry
	 * @param key A string used to retrieve the object
	 * @param widget the DbbWidget to insert
	 * @throws DbbXMLException
	 */
	public void putWidget( String key, DbbWidget widget ) throws DbbXMLException {
		 /* Design by Contract */
		 if ((key == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("key attribute is null"));
		 if ((widget == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("obj attribute is null"));
		/* DbC */
		if( registry.containsKey(key ) == true )
				throw new DbbXMLException(MsgManager.errmsg( "duplicate key: " + key ));
	            
		registry.put( key, widget );
	}//putElement
}
