package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlJoinInfo.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

import java.util.Vector;

import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**
 * Class to handle join structures.
 *
 * @author Carlo Boarotto, ESO 11-Sep-1999
 * 
 */
public class DbbSqlJoinInfo {

    /** List of database tables */
    protected Vector tables_;

    /** List of sql clauses */
    protected Vector clauses_;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create a DbbSqlJoinInfo object initializing its internal
     * variables 
     */
    public DbbSqlJoinInfo() {
	tables_ = new Vector();
	clauses_ = new Vector();
    }


    /**
     * Create a DbbSqlJoinInfo object initializing its internal
     * variables with the given values.
     *
     * @param tables_vector  List of database tables
     * @param clauses_vector List of sql clauses
     */
    public DbbSqlJoinInfo(Vector tables_vector,
			  Vector clauses_vector) {

	tables_ = tables_vector;
	clauses_ = clauses_vector;
    }

    //----------------------------------------
    // Public methods
    //----------------------------------------

    /** @return the tables in the tables_ list */
    public DbbSqlTable[] getTables() {

	int len = tables_.size();
	DbbSqlTable[] ret = new DbbSqlTable[len];
	tables_.copyInto(ret);
	return ret;
    }


    /** @return the sql clauses in the clauses_ list */
    public String[] getClauses() {
	int len = clauses_.size();
	String[] ret = new String[len];
	clauses_.copyInto(ret);
	return ret;
    }


    /**
     * Merge the tables coming from an other DbbSqlJoinInfo object
     * into the current instance.
     *
     * @param info DbbSqlJoinInfo object to be merged 
     */
    public void mergeTables(DbbSqlJoinInfo info) {

	DbbSqlTable[] array = info.getTables();
    
	for(int i=0; i<array.length; i++) {
      
	    if(!isContainedTable(array[i]))
		tables_.addElement(array[i]);
	}
    }


    /**
     * Merge the sql clauses coming from an other DbbSqlJoinInfo
     * object into the current instance.
     *
     * @param info DbbSqlJoinInfo object to be merged 
     */
    public void mergeClauses(DbbSqlJoinInfo info) {

	String[] array = info.getClauses();
    
	for(int i=0; i<array.length; i++) {
	    if(!isContainedClause(array[i]))
		clauses_.addElement(array[i]);
	}
    }


    /**
     * Check if a database table is already contained in the internal
     * list.
     *
     * @param table Database table
     * @return Check result 
     */
    public boolean isContainedTable(DbbSqlTable table) {
	return tables_.contains(table);
    }


    /**
     * Check if a sql clause is already contained in the internal
     * list.
     *
     * @param clause Sql clause
     * @return Check result 
     */
    public boolean isContainedClause(String clause) {
	return clauses_.contains(clause);
    }


    /**
     * Merge the data contained in a vector of DbbSqlJoinInfo
     * objects into the current instance.
     *
     * @param info_vector DbbSqlJoinInfo objects to be merged 
     */
    public static DbbSqlJoinInfo merge(Vector info_vector) {

	DbbSqlJoinInfo new_info = new DbbSqlJoinInfo();

	int len = info_vector.size();

	for(int i=0; i<len; i++) {

	    DbbSqlJoinInfo current = 
		(DbbSqlJoinInfo)(info_vector.elementAt(i));
	    new_info.mergeTables(current);
	    new_info.mergeClauses(current);
	}

	return new_info;
    }


    /**
     * Dump the contents of the current instance.
     * @return A string containing the current instance's data
     */
    public String toString() {
	String ret ="";
	ret = ret + "Tables: " + tables_ + "\n";

	int len = clauses_.size();

	for(int i=0; i<len; i++)
	    ret = ret + "Joins:  " + clauses_.elementAt(i) + "\n";

	return ret;
    }
}
