package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of an epoch.
 * 
 * <p>
 * $Id: AirmassModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 * @author cmavros 29-10-01 Add inserString()
 */

public class AirmassModel extends InputMaskModel {
    
    private final float AIRMASS_MAX = 5;
    private final float AIRMASS_MIN = 1;

    /**
     * The regular expression describing legal buffer values.
     * 
     */
    private final static String legalValuesPattern_ =
	"\\d{0,1}(\\.\\d{0,1})?";

    /**
     * Constructor for the model.
     */
    public AirmassModel() {
	super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();
            if (buffer_val.equals(".")){
                remove(offset, str.length());
            }
            else{
                Float f = Float.valueOf(buffer_val);
                if(f.floatValue() < AIRMASS_MIN) {
                    remove(offset, str.length());
                }
                else if(f.floatValue() > AIRMASS_MAX) {
                    remove(offset, str.length());
                }
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}

