package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;


/**
 * A Document model to only permit the input of a Duration.
 * in the format of 31:21:30:00
 * <p>
 * $Id: ISODateModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */

public class ISODateModel extends InputMaskModel {

    private Character xDash = new Character('-');
    private Character xCol = new Character(':');
    private Character xT = new Character('T');

    /**
     * Legal values if they are valid
     * looking for a Date in an ISO date layout yyyy-MM-dd'T'HH:mm:ss
     */

    private final static String legalValuesPattern_ = 
        "^(\\d{0,4}(-\\d{0,2}(-\\d{0,2}(T\\d{0,2}(:\\d{0,2}(:\\d{0,2})?)?)?)?)?)?";

    /**
     * Constructor for the model.
     */
    public ISODateModel() {
	super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     * 
     * <br>ChangeLog: cmavros 15-11-01
     */
    public void insertString(int offset, String str, AttributeSet a)
        throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();
            //Do not accept a single :
            if (buffer_val.equals("-")){
                remove(offset, str.length());
            }
            else {
                int len = buffer_val.length();
                //stdlog_.debug("len: <"+len+">");
                if(len == 1 && (Integer.parseInt(str) != 2 ))
                    remove(offset, str.length());
                if (len > 1 ){
                    /** Check for double - or : or a : after the T*/
                    Character x = new Character(buffer_val.charAt(len-2));
                    if (x.equals(xDash) && str.equals("-"))
                        remove(offset, str.length());
                    else if (x.equals(xCol) && str.equals(":"))
                        remove(offset, str.length());
                    else if (x.equals(xT) && str.equals(":"))
                        remove(offset, str.length());
                } 

                if (len > 1 && len < 5){
                    /** Check year */
                    if(len == 2 && Integer.parseInt(str) != 0 )
                       remove(offset, str.length());
                    if(len == 3 && Integer.parseInt(str) > 3)
                       remove(offset, str.length());
                    Float f = Float.valueOf(buffer_val);
                    if(f.floatValue() > 2039) 
                            remove(offset, str.length());
                    
                }
                if (len > 5 && len < 8){
                    /** Check day */
                    if(len >= 7){
                        Float  f = Float.valueOf(buffer_val.substring(5,7));
                        if(f.floatValue() > 31 || f.floatValue()==0) 
                           remove(offset, str.length());
                    }
                }

                /** Check if only a sinlge digit was entered for day */
                else if (len > 7 && len < 10){
                    Character x = new Character (buffer_val.charAt(6));
                    if (x.equals(xDash)){
                        /** Check month */
                        if(len >= 9 ){
                            Float f = Float.valueOf(buffer_val.substring(7,9));
                            if(f.floatValue() > 12 || f.floatValue()==0) 
                               remove(offset, str.length());
                        }
                    }
                }

                else if (len > 8 && len <11) {
                    /** Check month */
                    if(len >= 10){
                        Float f = Float.valueOf(buffer_val.substring(8,10));
                        if(f.floatValue() > 12 || f.floatValue()==0) 
                           remove(offset, str.length());
                    }
                }
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}

