package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ScrollPaneConstants;

import org.apache.log4j.Logger;

/**
 * This class creates two list boxes: the one on the left contains a
 * list of entries. Every time an item is selected the right list box
 * is updated with values referring to the selected item.
 * 
 *  @see org.eso.ohs.core.gui.models.SubListModel
 */

public class SingleListBox extends JDialog  {

	private static final String rcsid =
		"$Id: SingleListBox.java,v 1.5 2004/04/08 12:09:51 tcanavan Exp $";

	private static Logger stdlog_ = Logger.getLogger(SingleListBox.class);

	/**
	 * OptionPane status
	 */

	private int returnValue_ = JOptionPane.CLOSED_OPTION;

	/**
	 * label above main column
	 */

	private JLabel label1_ = new JLabel();

	/**
	 * List to hold main data
	 */
	private JList main_ = new JList();

	
	/** 
	 * Main data model
	 */

	private DefaultListModel mainModel_;

	
	/**
	 * Button to commit changes
	 */

	private JButton ok_ = new JButton();

	/**
	 * Button to cancel changes
	 */

	private JButton cancel_ = new JButton();

	/**
	 * GridBagConstraints
	 */

	private GridBagConstraints c_ = new GridBagConstraints();

	/** 
	 * Panel to add components to.
	 */

	private JPanel panel_ = new JPanel();

	/**
	 * Create a NestedListBox Dialog box
	 * @param f Frame of Dialog box
	 * @param mainModel model for main data
	 * @param subModel model for desination list 
	 */

	public SingleListBox(JFrame f, DefaultListModel mainModel) {
		super(f, "Export", true);
		mainModel_ = mainModel;
		
		main_.setModel(mainModel_);
		//main_.addListSelectionListener(this);
		//main_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		main_.setValueIsAdjusting(false);
		
		init();
		setContentPane(panel_);
		pack();
	}

	
	/**
	 * Main Model
	 * @return main model
	 */

	public DefaultListModel getMainModel() {
		return mainModel_;
	}

	/**
	 * Return Value used by OptionPane
	 * 
	 * @return returnValue
	 */

	public int getValue() {
		return returnValue_;
	}

	/**
	 * Set the main model for the main list
	 * @param mainData data to be copied into mainModel
	 */

	public void setMainModel(Vector mainData) {
		for (int i = 0; i < mainData.size(); i++)
			mainModel_.addElement(mainData.elementAt(i));
	}

	/**
	 * Set the main model for the main list
	 * @param mainData data to be copied into mainModel
	 */

	public void setMainModel(Object[] mainData) {
		for (int i = 0; i < mainData.length; i++)
			mainModel_.addElement(mainData[i]);
	}

	
	/**
	 * Return the selected entry in the main list
	 * @return the selected entry in the main list
	 */

	public String getSelMainValue() {
		return (String) main_.getSelectedValue();
	}

	/**
	 * Return an array containing the selected values in the main list
	 * @return the selected entries in the sub list
	 */

	public String[] getSelMainValues() {

		Object[] values = main_.getSelectedValues();
		String[] stringValues = new String[values.length];

		for (int i = 0; i < values.length; i++) {
			stringValues[i] = (String)values[i];
		}
		return stringValues;
	}

	/** 
	 * Set the cell renders for the main and subl list
	 * 
	 * @param renderer renderer for items in sub and main lists
	 */
	public void setCellRenderers(ListCellRenderer renderer) {
		main_.setCellRenderer(renderer);
		//sub_.setCellRenderer(renderer);
	}

	/**
	 * Set the title for the main list
	 * @param title title for main list 
	 */
	public void setMainTitle(String title) {
		label1_.setText(title);
	}

	/**
	 * Inirialise the member variables and add gui components 
	 * to the frame.
	 */
	private void init() {
		main_.setModel(mainModel_);
		
		main_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
		//panel_.setLayout(new GridBagLayout());
	
		panel_.setLayout(new BoxLayout(panel_, BoxLayout.Y_AXIS));

        JPanel listPanel = new JPanel();
		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
	
		JPanel buttonPanel = new JPanel();
				
	   
		label1_.setText("Main list");
		final JScrollPane mainScrollPane =
			new JScrollPane(
				main_,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		label1_.setHorizontalAlignment(JLabel.LEFT);
		mainScrollPane.setPreferredSize(new Dimension(450, 110));

		listPanel.add(label1_);
		listPanel.add(mainScrollPane);
		panel_.add(listPanel);
		
		initOKCancel(buttonPanel);
		panel_.add(buttonPanel);
	
	}

	/**
	 * Add listeners for Dialog buttons
	 */

	public void setOkAction(ActionListener okAction) {

		ok_.addActionListener(okAction);
	}

	public void setCancelAction(ActionListener cancelAction) {

		cancel_.addActionListener(cancelAction);
	}

	public void setOkLabel(String okLabel) {
		ok_.setText(okLabel);
	}

	public void setCancelLabel(String cancelLabel) {
		cancel_.setText(cancelLabel);
	}

	/**
	 * Initialise ok and cancl buttons and there grid bag constraints
	 */

	private void initOKCancel(JPanel buttonPanel) {
		addOKButton(buttonPanel);
		addCancelButton(buttonPanel);
	}

	/**
	 * Add OK button to panel
	 */

	public void addOKButton(JPanel buttonPanel) {
		ok_.setText("Save");
		buttonPanel.add(ok_);
	}

	/**
	 * Add Cancel button to Panel
	 */

	public void addCancelButton(JPanel buttonPanel) {
		cancel_.setText("Cancel");
		buttonPanel.add(cancel_);
	}

//	public void valueChanged(ListSelectionEvent e) {
//
//		if (!e.getValueIsAdjusting()) {
//			stdlog_.debug(e);
//			String proposalCode = (String) main_.getSelectedValue();
//			if (proposalCode != null) {
//				subModel_.updateList(proposalCode);
//			}
//		}
//	}
}
