package org.eso.ohs.dfs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.commons.collections.LRUMap;
/**
 * This class caches finding chart data for local storage purposes only.
 * This is to avoid the duplication of the finding chart data acrosss
 * multile OB's files. The fcData variable in the FindingChart class is    
 * only initialised when sending data to the appserver. So
 * in the event of this variable being null we call the FindingChartCache and 
 * return the data from the cache. This fcData variable does *not* get initialised  
 * automatically when dealing with the RDBMS this is to avoid unecessary streaming
 * of finding chart data when reading an OB.
 */


public class FindingChartCache {
    
    


     private static LRUMap inMemory_ =  new LRUMap();




     /**
      * Get the finding chart data and cache the data.
      * @param key File CRC to identify elements in CACHE 
      * @param fileName The location of the file on disk
      * @return byte[] Data in file.
      * @throws IOException
      */

 
     public static byte []  getFindingChartData(long key,String fileName) throws IOException  {
         FindingChartData fcData=new FindingChartData();
         Long checkSum = new Long(key);
         if(inMemory_.containsKey(checkSum)) {
             fcData = (FindingChartData)inMemory_.get(checkSum);              
         }           
         File file = new File(fileName);
         if(file.exists()) {
             FileInputStream fileInput =
                     new FileInputStream(file);
             fcData.fcBytes =  new byte[(int) file.length()];             
             fileInput.read(fcData.fcBytes);
             inMemory_.put(checkSum,fcData);
             return fcData.fcBytes;
         } else {
             throw new FileNotFoundException("File not found " + fileName);
         }         
         
     }
     
     
     public static void setMapSize(int size) {
         inMemory_.setMaximumSize(size);
     }
     
     
     private static class FindingChartData {
         
         
           private byte [] fcBytes;
                      
           
           
     }         
}
    
    
