package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for OBs stored in the database.
 *
 * @version $Revision: 1.1 $
 * @author Tim Canavan
 *
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerOR extends DbaseIOHandler {

    /** Standard log channel */
    private static Logger stdlog_ =
	Logger.getLogger( DbaseHandlerOR.class );
    
    private static final String rcsid = "$Id: DbaseHandlerOR.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $"; 

    private static String opcDB_ = Config.getCfg().getDbName(
					       ObservingRun.getSuffix());
					     
	/**
	 * This is the lowest period number for which IP versioning info
	 * is expected. All observing runs from this period number onwards
	 * must contain an IP version for they will not be loaded from the
	 * database (they are not valid).
	 * If left as 0, the value of IP version is ignored.
	 */
	private int ipVersioningStartPeriod_ = 0;

    /**
     * Constructor for OB I/O class.
     */
    public DbaseHandlerOR() {
        ipVersioningStartPeriod_ = Config.getCfg().getStartPeriodForIPVersioning();
    }

    /**
     * Managed class
     * @return Observing Run class
     */

    public Class getManagedClass() {
	return ObservingRun.class;
    }

   /**
     * Read data of an Observing Run.
     *
     * @param stmt	SQL statement
     * @param objId	Object id to be read
     *
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database   
     * @exception ObjectIOException I/O Error occured while reading from 
     *				    the database.
     */

    public UnpackedStorableObject read( Statement stmt, long objId )
     	throws ObjectIOException, ObjectNotFoundException, SQLException {
     
             Vector obrs = new Vector();
     	ObservingRun or = (ObservingRun)newInstance();
     	UnpackedStorableObject uso = new UnpackedStorableObject(or);
     	Config cfg = Config.getCfg();    
     
     	// @author amchavan, 29-May-2002: fixed 1758, reformatted
     	String sql = 
     	    "SELECT DISTINCT " +
     
     	    "runs.id, " +
     	    "runs.rank, " +
     	    "schd.nights, " +
            "prog.gto, " +            
     	    "schd.rank_class, " +
     	    "runs.programme_id, " +
     	    "runs.run_desc, " +
     	    "schd.inst AS inst_code, " +
             "schd.ipversion, " +
     	    "schd.obs_mode, " +
     	    "schd.tel AS tel_code, " +
     	    "schd.progid, " +
            "usrs.item_name AS firstname, " +
            "usrs.surname_dl AS surname_dl, " +
     	    "prog.title, " +
     	    // "prog.period " +
            // resolves bug 2121.
            "schd.period " +
     
     
     	    fromClause() +
            " WHERE " + "runs.id = " + objId +
            //" AND " + "usrs.id = " + getUserId() +     
     	    
     	    joinConditions();
     
     	stdlog_.debug( sql );
             ResultSet rs = stmt.executeQuery(sql);
     
     	if ( ! rs.next()) { // Only one OR with this id !!
     	    throw new ObjectNotFoundException("Object not found in database");
     	}
     
        String[] instNames = InstrumentList.getInstance().getInstrumentNames();
     	or.setInstrumentList(instNames);
     	int obr_id = rs.getInt("id");
     	long id = cfg.tableToUniqueId(obr_id, ObservingRun.class);
     	or.setId(id);
     	or.setRank(rs.getShort("rank"));
        or.setGto(rs.getShort("gto"));
     	or.setRankClass(rs.getString("rank_class"));
     	or.setInstCode(rs.getString("inst_code"));
        or.setPIName(rs.getString("firstname") + " " + rs.getString("surname_dl"));            
     	float ipv = rs.getFloat("ipversion");
     	if (ipv <= 0) {
     	   ipv = 0;
         }
         or.setIPVersion(ipv);
         
     	or.setObsMode(rs.getString("obs_mode"));
     	or.setTelescope(rs.getString("tel_code"));
        if(or.isServiceMode()) {
             or.setDecimalHours(rs.getFloat("nights")); // visitor mode = days service mode = hours.
        }     
        else {
             or.setDecimalNights(rs.getInt("nights"));            
        }
     	or.setTitle(rs.getString("title"));
     	or.setPeriod(rs.getInt ("period"));
     	or.setProgrammeId(rs.getInt("programme_id"));
     	or.setRunDescription(rs.getString("run_desc"));
     	or.setProgId(rs.getString("progid"));
     
     	if (or.getObsMode() == null) {
     	    throw new ObjectIOException(
     			"CONTACT ESO: bad Observing Run in database: no mode: "
     			+ objId
     		    );
     	}
     	
        /* 
         * Invalid check as the OR period is taken from obs_programmes
         * not from sched_rep and thus may be different (IP version is
         * assoc with sched_rep period only).
         *
         if (ipVersioningStartPeriod_ > 0
                 && or.getPeriod() >= ipVersioningStartPeriod_
                 && or.getIPVersion() == 0) {
             throw new ObjectIOException(
                 "Object has invalid IP version and cannot be loaded.");
         }
         */
     
     	or.setName(
     	    or.getProgId()
     	    + "/" + or.getObsMode().toUpperCase() + "M"
     	    + "/" + or.getInstCode()
     	);
     
     	rs.close();
        stdlog_.debug("ObservingRun Created id:"+objId+" Period:"+or.getPeriod()
            +" Inst:"+or.getInstCode()+" IPVersion:"+or.getIPVersion());
        //(new Exception()).printStackTrace();
     	return new UnpackedStorableObject(or);
    }

    /**
     * Not required
     * @exception SQLException not required
     */

    public void write(Statement stmt, long id, UnpackedStorableObject obj)
      throws SQLException {
	// stdlog_.debug("write: Writing data");
    }

    /**
     * Not required
     * @exception SQLException not required
     */

    public void update(Statement stmt, long id, UnpackedStorableObject obj)
      throws SQLException {
	// stdlog_.debug("update: Reading data");
    }

    /**
     * Not required
     * @exception SQLException not required
     */

    public void delete(Statement stmt, long id)
      throws SQLException {
	// stdlog_.debug("delete: Deleting data");
    }


    /**
     * Query the database for the Observing Runs of a specific user.
     *
     * @return A list of summaries, each summary applies to an Observing Run.
     * 
     * @param	stmt	JDBC Statement for querying the dbase.
     * 
     * @exception ObjectIOException
     *		    an error occurred during the dbase operation
     */
    public Summary[] listObjects(Statement stmt)
				throws ObjectIOException {
	Vector sumVec = new Vector();

	
    String ipversionFilter = ipVersioningStartPeriod_ > 0 ?
        " AND ("  + "schd.period < "+ipVersioningStartPeriod_ +
            " OR schd.ipversion IS NOT NULL ) " : "";
    
    // @author amchavan, 29-May-2002: fixed 1758, 1752        
	String sql =
	    "SELECT DISTINCT " +
	    
	    "runs.id, " +
	    "prog.period, " +
	    "prog.title " +

	    fromClause() +

	    " WHERE " + "prop.user_id = " + getUserId() +
	    " AND "   + "prop.pi_flag = 0 " +
	    " AND "   + "schd.published = 1 " +
	    // MIPS - only return IPs after the start period which have an assigned ipversion.
	    ipversionFilter +
	    
	    joinConditions();

        System.out.println( sql );

	stdlog_.debug( sql );

	try {
	    ResultSet rs = stmt.executeQuery(sql);

	    while(rs.next()) {
		Summary sum = new Summary();

		sum.setObjectClass(ObservingRun.class);

		sum.setId(
		    Config.getCfg().tableToUniqueId(
			rs.getLong("id"),
			ObservingRun.class
		    )
		);

		sum.setPropertyValue("Id", new Long(sum.getId()));

		sum.setPropertyValue("period", rs.getObject("period"));
		sum.setPropertyValue("title", rs.getObject("title"));

		sumVec.addElement(sum);
	    }
	    rs.close();

	}
	catch (SQLException e) {
	    e.printStackTrace();
	    throw new ObjectIOException(e);
	}

	Summary[] result = new Summary[sumVec.size()];
	sumVec.copyInto(result);
	return result;
    }


    /** @return A standard "FROM" clause for the Phase I SQL queries 
     *
     * <br>ChangeLog: amchavan, 29-May-2002: added
     */
    private String fromClause() {
	
	String sql =
	    " FROM " +
	    opcDB_ + "..obs_runs runs, "  +
	    opcDB_ + "..sched_rep schd, "  +
	    opcDB_ + "..obs_programmes prog, "  +
	    opcDB_ + "..proposed prop," +
        opcDB_ + "..eso_users usrs ";

	return sql;
    }



    /** @return A set of "AND" clauses for the Phase I SQL queries.
     *          The clauses define <em>only</em> the join conditions for
     *          the tables returned by <tt>fromClause</tt>.
     *
     * <br>ChangeLog: amchavan, 29-May-2002: added
     * @see #fromClause()
     */
    private String joinConditions() {
	
	String sql =

	    // obs_runs <-> sched_rep
	    " AND runs.id = schd.run_id" +

	    // obs_programmes <-> obs_runs
	    " AND runs.programme_id = prog.id " +

	    // proposed <-> obs_programmes
	    " AND prog.id = prop.programme_id " +
        
        " AND prop.pi_flag = 0 " + 
        
        " AND usrs.id = prop.user_id";


	return sql;
    }
}


