package org.eso.ohs.persistence.dbase.phase2;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.PixelToMM;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.MaskEvent;
import org.eso.ohs.persistence.MaskStatusListener;


/**
 *This class provides a set of static method in order to
 *interact with the phase2 database
 *
 * @version $Revision: 1.3 $
 * @author ?
 */

public class MaskQueueManager {

    /** Standard log channel */
    private static Logger stdlog_ =
	Logger.getLogger( MaskQueueManager.class );
    

    private static final String rcsid = "$Id: MaskQueueManager.java,v 1.3 2005/09/05 15:09:08 ddorigo Exp $"; 

    
 protected static String obrepDbName_ = Config.getCfg().getDbName(
                                       ObservationBlock.getSuffix());



  public static String tmpDB    ="tempdb";
  //  public static String exec_tmp = tmpDB + "..tmp_exec_sequences";
  public static String exec_tmp = "#tmp_exec_sequences";
  public static String sch      ="schedule_items";
  public static String exec     ="exec_sequences";

  //  public static Connection conHack = null;

  private static EventListenerList listenerList = new EventListenerList();



  /** Adds a listener for an OBEvent */
      public static void addMaskListener(MaskStatusListener l) {
          listenerList.add(MaskStatusListener.class, l);
      }

     /** Remove a listener for an OBEvent */
     public static void removeMaskStatusListener(MaskStatusListener l) {
         listenerList.remove(MaskStatusListener.class, l);
     }

     /** Notify all listeners that are registered for a OBEvent */

     public static void fireMaskPropertyChange(MaskEvent e) {
         Object[] listeners = listenerList.getListenerList();
         MaskStatusListener lnr = null;
         for (int i=0; i < listeners.length; i+=2) {
             lnr = (MaskStatusListener)listeners[i+1];
             lnr.maskPropertyChange(e);
         }      
     }

   

  

   public static void setMaskRowStatus(Vector maskIdList,String status,String dummyNaid) {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;  
        try {
            conn = pool.acquireConnection(Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                                          Config.getCfg().getDbUserName(), 
                                          Config.getCfg().getDbPassword());
            conn.setAutoCommit(false);
            stmt = conn.createStatement();
            StringBuffer buf = new StringBuffer();
            for(int j=0;j<maskIdList.size();j++) {	        
                if(j == 0)
                    buf.append( '(' );
                buf.append( '\'' );
                buf.append(maskIdList.elementAt(j));              
                buf.append( '\'' );
                if(j == maskIdList.size()-1) {
                    buf.append( ')' );                   
                }
                else
                    buf.append(",");
            }
            if(buf.length() >0) {           

                String sql = "UPDATE " + obrepDbName_ + "..masks set mask_status  = " + "'" + status + "'" + 
                " where " + obrepDbName_ + "..masks.bar_code IN " +  buf.toString() +
                " AND  " + obrepDbName_ + "..masks.mask_naid != '" + dummyNaid + "'";
                System.out.println(sql);
                stmt.executeUpdate(sql);	    
                conn.commit();
            }
        }
        catch (SQLException e) {
            try {
                conn.rollback();
            }
            catch (SQLException exc) {
                exc.printStackTrace();
            }
            e.printStackTrace();
	 
        }
        finally {
            try {
                conn.setAutoCommit(true);
            }
            catch(SQLException e) {
            }
            ConnectionPool.getPool().releaseConnection(conn);
        }
        fireMaskPropertyChange(new MaskEvent("MaskStatus",status));

    }





  

 public static void setMaskStatusFromNaid(Vector naidAdpList,String status,int cabinetCount) throws NumberFormatException {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;  
        try {
            conn = pool.acquireConnection(Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                                          Config.getCfg().getDbUserName(), 
                                          Config.getCfg().getDbPassword());
            conn.setAutoCommit(false);
            stmt = conn.createStatement();
            StringBuffer buf = new StringBuffer();
            for(int j=0;j<naidAdpList.size();j++) {     
                if(j == 0)
                    buf.append( '(' );
                buf.append( '\'' );
                buf.append(((NaidAdp)naidAdpList.elementAt(j)).naid);              
                buf.append( '\'' );
                if(j == naidAdpList.size()-1) {
                    buf.append( ')' );                   
                }
                else
                    buf.append(",");
            }
            if(buf.length() >0) {           
                String sql = "UPDATE " + obrepDbName_ + "..masks set mask_status  = " + "'" + status + "'" + 
                    " where " + obrepDbName_ + "..masks.mask_naid IN " +  buf.toString();
                System.out.println(sql);
                stmt.executeUpdate(sql);	    
               
            }
            int maskNo=-99;
            String sql = "SELECT  MAX(convert(int,bar_code) % 100000) from " + obrepDbName_ + "..masks";
            ResultSet rs = stmt.executeQuery(sql);
            while(rs.next()) {
                maskNo  = rs.getInt(1);                 
            }
            if(maskNo >= 0)
                maskNo++;
            else
                maskNo=1;
            String dateTime = TextUtils.currentISODateTime();
  
            long old_obid = ((NaidAdp)naidAdpList.elementAt(0)).ob_id;         
            for(int i=0;i<naidAdpList.size();i++) {                             
                String naid = ((NaidAdp)naidAdpList.elementAt(i)).naid;
                int quad = ((NaidAdp)naidAdpList.elementAt(i)).quad;
                long ob_id = ((NaidAdp)naidAdpList.elementAt(i)).ob_id;
                Parameter [] pList = ((NaidAdp)naidAdpList.elementAt(i)).pList;
                if(old_obid != ob_id) {
                    old_obid = ob_id; 
                    maskNo++;
                    dateTime = TextUtils.currentISODateTime();
                }
                int maskId = (100000 * quad) + maskNo;
                PafFile adpFile = ((NaidAdp)naidAdpList.elementAt(i)).adpFile;
                PixelToMM pixtoMM = null;
                try {
                    pixtoMM = new PixelToMM(adpFile,maskId,dateTime);                                       
                } catch(NumberFormatException e) {
                    stmt.close();
                    conn.rollback();
                    conn.setAutoCommit(true);
                    ConnectionPool.getPool().releaseConnection(conn);
                    throw e;                    
                }
                String admString = pixtoMM.createAdmString();
                sql = "UPDATE " + obrepDbName_ + "..masks set bar_code = "  
                + SqlUtils.sqlEscapedString(String.valueOf(maskId)) + 
                ", adm_file = " +
                SqlUtils.sqlEscapedString(admString) +
                " where " + obrepDbName_ + "..masks.mask_naid = " +
                SqlUtils.sqlEscapedString(naid);
                System.out.println(" SQL UPDATE " + sql);        
                stmt.executeUpdate(sql);
                for(int j=0;j<pList.length;j++) {
                    if(pList[j].getParameterName().equals("INS.MASK" + quad + ".ID")) {
                        pList[j].setValue(new Integer(maskId).toString());
                    }
                }

           }   
           conn.commit();     
        }
        catch (SQLException e) {
            try {
                conn.rollback();
            }
            catch (SQLException exc) {
                exc.printStackTrace();
            }
            e.printStackTrace();
	 
        }
        finally {
            try {
                conn.setAutoCommit(true);
            }
            catch(SQLException e) {
            }
            ConnectionPool.getPool().releaseConnection(conn);
        }
        fireMaskPropertyChange(new MaskEvent("MaskStatus",status));
    }
 public class NaidAdp {
     private String naid;
     private PafFile adpFile;
     private int quad = -1;
     private long ob_id;
     private Parameter[] pList;
     
     
     public NaidAdp(
         String naid,
         PafFile adpFile,
         int quad,
         long ob_id,
         Parameter[] pList) {
         this.naid = naid;
         this.adpFile = adpFile;
         this.quad = quad;
         this.ob_id = ob_id;
         this.pList = pList;
     }

   public String getMaskName() {
       return adpFile.getPafName();
   }

 }

}



  


    
   







