package org.eso.ohs.phase2.actions;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OverwriteOptionPane;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Export;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to create a business object in the chosen directory.
 * 
 * <p>
 * $Id: ExportBOAction.java,v 1.16 2005/02/11 11:00:27 tcanavan Exp $
 * @author Tim Canavan
 * <br>ChangeLog: cmavros 2002/JUNE/03 [1766/P2PP], Do not allow export of OBs with
 *   blank name.
 */

public class ExportBOAction extends ActionSuperclass {
    /** medium on which the business object is to be created */
    private Media device_;

    /** view of the folder in which the OB is to be created */
    private ObjectContainer folderView_;

    private boolean version2_;

    private int returnValue_;

    private boolean allOBs_ = false;

    private static final int YES_OPTION = 0;
    private static final int NO_OPTION = 1;
    private static final int YES_TO_ALL_OPTION = 2;
    private static final int CANCEL_OPTION = 3;

    private static SwingFileChooser fileChooser_ = new SwingFileChooser();
    static {
        File f1 = new File(AppConfig.getAppConfig().getExpDir());
        fileChooser_.setCurrentDirectory(new File(f1.getParent()));
        fileChooser_.setSelectedFile(f1);
    }
    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public ExportBOAction(
        Media device,
        ObjectContainer folderView,
        String name,
        boolean version,
        boolean allOBs) {
        super((JComponent) folderView, name);
        version2_ = version;
        device_ = device;
        folderView_ = folderView;
        allOBs_ = allOBs;
    }

    /**
        * Constructs a create BusinessObject action.
        * 
        * @param device      medium containing the parent directory.
        * @param folderView  GUI view of (summary information in) the folder.
        */
    public ExportBOAction(
        Media device,
        ObjectContainer folderView,
        String name,
        boolean version) {
        super((JComponent) folderView, name);
        version2_ = version;
        device_ = device;
        folderView_ = folderView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long[] idLongs;
        if (allOBs_)
            idLongs = folderView_.getAll();
        else
            idLongs = folderView_.getAllSelected();
        Object[] options = { "Yes", "No", "Cancel" };

        if (idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected objects.",
                    "Which objects should be Exported ?",
                    },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }

        fileChooser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        
        int response =
            fileChooser_.showDialog(null, "Select Export Directory ");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;
        if (response == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();
            if (!f.canWrite()) {
                JOptionPane.showMessageDialog(
                    null,
                    new Object[] { "No write permissions to export to this directory" },
                    "No write permisssions",
                    JOptionPane.ERROR_MESSAGE);
            } else {

                new IncrementalActionAutomaton(
                    new IncrementalExport(
                        folderView_.getMedia(),
                        f.getAbsolutePath(),
                        idLongs),
                    (JFrame) folderView_.getTopLevelAncestor(),
                    "Exporting...");

            }
        }
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalExport implements IncrementalAction {

        /** cached reference to Persistence */
        private ObjectManager objMgr = ObjectManager.getObjectManager();

        /** array of ids of objects to be Exported */
        private Long[] idLongs_;

        /** Media containing the objects to be Exported */
        private Media device_;

        /** Directory containing the ObsBlocks */
        private String dir_;

        Export export_ = new Export();

        /**
         * Constructor.
         * 
         * @param frm	    window on which to base any dialog boxes.
         * @param device    media containing the objects to be duplicated
         * @param dir	    directory containing the ObsBlocks
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalExport(Media device, String dir, Long[] idLongs) {
            device_ = device;
            dir_ = dir;
            idLongs_ = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
            returnValue_ = -1;
        }

        public void endOfAction() {
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return	    true if the duplicating should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                // Main body of the "loop" + fall thru
                doExport(idLongs_[counter].longValue());
            }
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id	    id of business object to duplicate
         */
        public void doExport(long id) {
            try {
                Class objType = Config.getCfg().getClassFromId(id);
                BusinessObject obj =
                    (BusinessObject) objMgr.getBusObj(device_, id, objType);
                String suffix = null;
                if (objType.equals(CalibrationBlock.class))
                    suffix = "ob";
                else
                    suffix = BusinessObject.getSuffix(objType);

                if (obj.getName() == null
                    || obj.getName().trim().length() == 0) {
                    JOptionPane.showMessageDialog(
                        folderView_.getTopLevelAncestor(),
                        new Object[] {
                            "You can not export a file without a name.",
                            "Please enter a name in the name field.",
                            },
                        "Blank Name",
                        JOptionPane.ERROR_MESSAGE);
                    return;
                }

                File file =
                    new File(
                        dir_
                            + File.separator
                            + obj.getName().replace(' ', '_')
                            + "."
                            + suffix
                            + "x");
                if (file.exists()
                    && returnValue_ != OverwriteOptionPane.YES_TO_ALL_OPTION) {
                    OverwriteOptionPane pane = new OverwriteOptionPane();
                    returnValue_ =
                        OverwriteOptionPane.showOptionDialog(
                            (JFrame) folderView_.getTopLevelAncestor(),
                            "Do you wish to overwrite previously exported file(s) "
                                + obj.getName().replace(' ', '_')
                                + "."
                                + suffix
                                + "x ? ",
                            "Overwrite files");
                }
                if (!file.exists()
                    || returnValue_ == OverwriteOptionPane.YES_OPTION
                    || returnValue_ == OverwriteOptionPane.YES_TO_ALL_OPTION) {
                    export_.export(dir_, obj, version2_,device_);
                }
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(
                    folderView_.getTopLevelAncestor(),
                    e);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(
                    folderView_.getTopLevelAncestor(),
                    e);
            } catch (RuntimeException e) {
                e.printStackTrace();
                throw e;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
