package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;

/**
 * Generate a report of Observation Blocks referencing missing or
 * obsolete templates.<BR>
 *
 * A template is <em>missing</em> if it is part of the OB, but it is
 * not available in any of the loaded Instrument Packages.<BR>
 * 
 * A template is <em>obsolete</em> if the Instrument Package version
 * string (i.e., the value of the "<code>VERSION</code> keyword in the
 * Instrument Summary File) is different from the value stored with
 * the OB (i.e., the value of the
 * <code>obrep2..obs_descriptions.ipversion</code> column for that OB).
 *
 * @version $Id: MissingTemplatesReport.java,v 1.13 2005/02/11 10:31:49 tcanavan Exp $
 *
 * @author amchavan, 21-Oct-2001: Creation from Stage2VerifyBOAction.java
 */

public class MissingTemplatesReport extends ActionSuperclass {

    protected ObjectContainer folderView_;

   /** Standard log channel */
   private static Logger stdlog_ =
           Logger.getLogger( MissingTemplatesReport.class );

    private Media device_;

    protected boolean folder_;

    protected StringBuffer repMiss_   = null;
    // not needed in MIPS protected StringBuffer repVers_   = null;

    protected static TextDisplayWidget reportLog_ 
        = new TextDisplayWidget("Templates report");

    /** Number of OBs with missing templates */
    protected int numBad_ = 0;	

    /** Number of OBs with no problems */
    protected int numGood_ = 0;	 

    /** Number of OBs with IP version mismatch */
    // not needed in MIPS protected int numVers_ = 0;	 

    /**
     * indicates whether the end of the iteration is reached or if we are
     * still processing the body of set of data upon which the iteration is
     * being performed
     */

    private boolean finished_ = false;

    private static String obrepDbName_ = Config.getCfg().getDbName(
					       ObservationBlock.getSuffix());

    /**
     * Constructs a MissingTemplatesReport action.
     */

    public MissingTemplatesReport( ObjectContainer folderView, 
				   boolean folder,
				   String name ) {
	super((JComponent) folderView, name );

	folderView_ = folderView;
        folder_ = folder;
    }

    /**
     * Event handler for the VerifyBOAction action.
     * 
     * @param evt       user-initiated ActionEvent to quit.
     */

    public void actionPerformedImpl(ActionEvent evt) {
	Long [] idLongs = null;

        numBad_ = 0;
        numGood_ = 0;
	//numVers_ = 0;

	ObjectManager mMgr = ObjectManager.getObjectManager();

        if (!folder_ ) {
            idLongs = folderView_.getAllSelected();
        } else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);

	if (idLongs == null || idLongs.length == 0) {
	    JOptionPane.showMessageDialog(
		folderView_.getTopLevelAncestor(),
		new Object[] {
		  "There is no selected Business Objects ",
		  "Which object should be verified?",
		},
		"Nothing selected",
		JOptionPane.ERROR_MESSAGE
	    );
	    return;		// nothing selected?
	}

	try {
	    Config          cfg = Config.getCfg();
	    ConnectionPool pool = ConnectionPool.getPool();
	    String       suffix = ObservationBlock.getSuffix();
	    Connection      con = pool.acquireConnection( cfg.getUrl( suffix ),
							  cfg.getDbUserName(),
							  cfg.getDbPassword());
	    Statement      stmt = con.createStatement();
	    new IncrementalActionAutomaton(
		new IncrementalMissingTemplates( stmt, idLongs ),
		(JFrame)folderView_.getTopLevelAncestor(), 
		"Verifying..." );
            pool.releaseConnection(con);
	}
	catch( SQLException e ) {
	    String message = "Error: " + e.getMessage();  
	    JOptionPane.showMessageDialog( folderView_.getTopLevelAncestor(),
					   message, 
					   "SQL error", 
					   JOptionPane.ERROR_MESSAGE );
	}
    }


    public class IncrementalMissingTemplates implements IncrementalAction {

        private ObjectManager objMgr = ObjectManager.getObjectManager();
        private Long[] idLongs_;
        private Class objType_;
	private Statement stmt_;

        public IncrementalMissingTemplates( Statement s, Long[] ids ) {
            idLongs_ = ids;
	    stmt_ = s;
        }

       /**
        * @param  int counter - will be passed to the iteration method
        * @return number of steps remaining
        */

        public int getNumStepsRemaining(int counter) {
            int rc = 0;
            if (!finished_) {
                rc = idLongs_.length - counter;
            }
            return rc;
        }

        public void startOfAction() {

            repMiss_   = new StringBuffer();
            // repVers_   = new StringBuffer();
            finished_ = false;
            reportLog_.clearTextArea();
            reportLog_.setVisible(false);
        }

        public void endOfAction() {
            showFinalMsg();
        }

       /**
        * @param counter Step counter.
        */
        public void performStep(int counter) {
            if(!finished_) {
                doVerify(idLongs_[counter]);
            }
            if(counter + 1 == idLongs_.length) {
                finished_ = true;
            }
        }

       /**
	    * Verify an Observation Block.
	    *
        * @param  id Id of the Observation Block to verify.
        */

        public void doVerify( Long id ) {

	    boolean foundInvalidTemplate = false; // assume this OB is good
	    boolean didVersionCheck = false; 	// need to check IP version

	    long ob_id = Config.getCfg().uniqueToTableId( id.longValue() );

	    // This SQL query will return all template
	    // names for our OB.
	    //----------------------------------------
	    String sql = "SELECT\n" +
		"        OBID   = ob.ob_id,\n" +
		"	 OBName = ob.item_name,\n" +
		"	 IPvers = od.ipversion,\n" +
		"	 Inst   = od.instrument,\n" +
		"	 TSName = ts.item_name\n" +
		"\n" +
		"FROM   " + obrepDbName_ +  "..template_signatures ts,\n" +
		"	" + obrepDbName_ +  "..obs_descriptions    od,\n" +
		"	" + obrepDbName_ +  "..obs_blocks          ob\n" +
		"\n" +
		"WHERE    ob.ob_id = " + ob_id + "\n" + 
		"AND      ts.od_id = ob.ob_id\n" +
		"AND      od.od_id = ob.ob_id\n" +
		"\n" +
		"ORDER by ts.ts_index";
	    
	    try {

		ResultSet rs = stmt_.executeQuery( sql );
		
		// For each template, check if we know 
		// anything about it: if not, add a row 
		// to the final report.
		//----------------------------------------
		while (rs.next()) {

		    TemplateSignature ts = null;

		    long     tInt = rs.getInt   ( "OBID"   );
		    String   tStr = rs.getString( "OBName" ).trim();
		    String obInst = rs.getString( "Inst"   ).trim();
		    String tsName = rs.getString( "TSName" ).trim();

            float ipversion = rs.getFloat("IPvers");

		    // Java has no format(), so we need to pad 
		    // strings ourselves to get nice printouts
		    String   obID = TextUtils.spacePadded( 10, tInt );
		    String obName = TextUtils.padString( tStr, 32 );

            Instrument inst = InstrumentList.getInstance().getInstrument(obInst, ipversion);
		    ts = TemplateSignatureFactory.getTemplate(inst, tsName);


		    if( ts != null ) {

			// This is a valid template
			// ----------------------------------------

			if( didVersionCheck == true )	// Did version check?
			    continue;			// YES: nothing to do

			didVersionCheck = true;	// remember we did check
			continue;		// nothing else to do
		    }
			

		    // This is *not* a valid template
		    //----------------------------------------

		    if( ! foundInvalidTemplate ) {

			// This is the first bad template 
			// for this OB: need to add a line to
			// the report

			repMiss_.append( obID );
			repMiss_.append( ' ' );
			repMiss_.append( obName );
			foundInvalidTemplate = true; // remember we added line
		    }

		    repMiss_.append( ' ' );
		    repMiss_.append( tsName );
		    
		    stdlog_.debug( obID + " " +
				obName + 
				obInst + ", " +
				tsName + ": BAD" );
		}

		if( foundInvalidTemplate ) {
		    numBad_++;
		    repMiss_.append( '\n' );
		}
		else     
		    numGood_++;
	    }
	    catch( SQLException e ) {
		String message = "SQL error: " + e.getMessage();  
		JOptionPane.showMessageDialog( folderView_.getTopLevelAncestor(),
					       message, 
					       "SQL error", 
					       JOptionPane.ERROR_MESSAGE );
	    }
	    catch( IOException e ) {
		String message = "I/O error: " + e.getMessage();  
		JOptionPane.showMessageDialog( folderView_.getTopLevelAncestor(),
					       message, 
					       "I/O error", 
					       JOptionPane.ERROR_MESSAGE );
	    }
	}


       /**
        * showFinalMsg shows a dialog box with information
        */

        public void showFinalMsg() {
	    
	    StringBuffer report = new StringBuffer();

	    report.append( "Missing/Mismatched Templates report " );
	    report.append( TextUtils.currentISODateTime() );
	    report.append( " (UT)\n");
	    report.append("==================================================\n\n");

            if( numBad_ > 0 ) {

		report.append( "The following OBs " );
		report.append( "reference unknown templates:\n" );
		report.append( repMiss_ );
		report.append( '\n' );
		report.append( '\n' );
	    }

        /**
         * Not needed in MIPS
            if( numVers_ > 0 ) {

		report.append( "The following OBs " );
		report.append( "have IP version mismatch " );
		report.append( "(<OB-vers>/<IP-vers>):\n" );
		report.append( repVers_ );
		report.append( '\n' );
		report.append( '\n' );
	    }
	     */

	    report.append( "Verified OBs:   " );
	    report.append( (numBad_ + numGood_) );
	    report.append( '\n' );

	    report.append( "Valid:          " );
	    report.append( numGood_ );
	    report.append( '\n' );

	    report.append( "Missing:        " );
	    report.append( numBad_ );
	    report.append( '\n' );

        /**
         * Not needed in MIPS
         *
	    report.append( "IP vers. mism.: " );
	    report.append( numVers_ );
	    report.append( '\n' );
	     */
	    report.append( '\n' );

	    // report.append( "Note that both valid OBs and OBs with missing " );
	    // report.append( "templates may have IP version mismatches.\n" );

            reportLog_.setTextArea( report.toString() );
            reportLog_.setSize( new Dimension( 700, 300 ));
            reportLog_.setVisible( true );
        }
    }
}
