package org.eso.ohs.phase2.apps.masktracker;

import java.io.IOException;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.MaskInstrument;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.MaskEvent;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO;
import org.eso.ohs.persistence.dbase.phase2.MaskQueueManager;
import org.eso.ohs.persistence.dbase.phase2.MaskQueueManager.NaidAdp;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;
import org.eso.ohs.phase2.apps.ot.icdVcs.OTOBChecker;

/**
 * Interface to the database for Mask updates that cannot go
 * through the ObjectManager.
 * 
 * <p>
 * $Id: MaskManufacturing.java,v 1.25 2005/03/30 11:53:56 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * @author Tim Canavan
 */
public class MaskManufacturing extends MaskActionGenerator {

   private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    private PafData[] identifierList_;

    private static MaskManufacturing instance_ = null;

    private int MAX_CABINETS = 10;

    private static Logger stdlog_ =
        Logger.getLogger(ConnectionPool.class);

    private boolean debug_ = false;

    private static TextDisplayWidget log_ = new TextDisplayWidget("Report");

    private String defaultAdpData_ =
        "PAF.HDR.START\nPAF.TYPE \"paramfile\" ;\nPAF.ID \"3277996668361\" ;\nPAF.NAME \"ThisIsADummyAdm.adm\" ;\nPAF.DESC \"ADM:\" ;\nPAF.CRTE.NAME \"gbr2adm.awk\" ;\nPAF.CRTE.DAYTIM \"2001-08-01T14:19:21.000\" ;\nPAF.LCHG.NAME \" ;\" ;\nPAF.LCHG.DAYTIM \" ;\" ;\nPAF.CHCK.NAME \" ;\" ;\nPAF.CHCK.DAYTIM\" ;\" ;\nPAF.CHCK.CHECKSUM \"00000\" ;\nPAF.HDR.END\nTPL.FILE.DIRNAME \"$INS_ROOT/$INS_USER/ADF\" ;\n";

    private String[] selectedNaidList_;

    private Vector naidAdpList_ = new Vector();
    private Vector cannotMan_ = new Vector();

    private Vector naidList_ = new Vector();
    private String buttons_ [] = {"Yes","No","Report"};

    private MaskManufacturing() {
    }

    public void setSelectedNaidList(String[] selectedNaidList) {
        selectedNaidList_ = selectedNaidList;
    }

    public boolean inSelectedNaidList(String naid) {
        System.out.println(
            ">>>>>>>>>>>>> Selected Naid List" + selectedNaidList_.length);
        for (int i = 0; i < selectedNaidList_.length; i++) {
            System.out.println(
                ">>>>>>>>>>>>> Selected Naid List" + selectedNaidList_[i]);
            if (selectedNaidList_[i].equals(naid))
                return true;
        }
        return false;
    }

    public static MaskManufacturing getInstance() {
        if (instance_ == null) {
            instance_ = new MaskManufacturing();
        }
        return instance_;
    }

    public String generateOrder(int maxNumOBs) {
        int totalCount = 0;
        try {
                     
            
            OTOBChecker chk =
                new OTOBChecker(maxNumOBs, true, Mask.PENDING_MANUFACTURED);
            SwingUtilities.invokeAndWait(chk);
            CalibrationBlock[] cbList = chk.getOBs();
            CalibrationBlock[] uniqueOBs = getIdenticalMaskObs(cbList);
            totalCount = MaskDBIO.getMaskStoredCount();  
            StringBuffer buf = new StringBuffer();
            String report = getNaidsForMasks(cbList);
            //if(report.length() == 0)
            //    buf.append("No masks will be manufactured for the obs selected");
            //else {
                buf.append(report);
                //}  
            QueueView queueView = MTViewManager.getInstance().getSelectedQueueView();
            if(queueView == null)
                return "";
            //Updater.propagateMaskInfo();
//Object message, int messageType, int optionType,
            String question  = "Are you sure you to send a manufacturing order for " + queueView.getQueue().getName() + " ?";
            
            SelectionDialog d = new SelectionDialog(queueView,question,"Mask Manufacturing",true,
                                buf.toString(),"Manufacturing Report Statistics");
            d.show();
            int value = d.getValue();
            if(value == JOptionPane.OK_OPTION || value == JOptionPane.CANCEL_OPTION) {
                d.setVisible(false);
                d.dispose();
            }
            if(value == JOptionPane.CANCEL_OPTION)
                return "";
            return optimiseMaskManufacturing(uniqueOBs, false);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch(StorageCabinetFullException e) {
                 
             JOptionPane.showMessageDialog(
                    null,
                    " Too many masks to fit in the storage cabinet \n"
                        + "The total that is currently stored is " + totalCount
                        + "\n Total number of maks stored and inserted and the number "
                        + "\n you wish to manufacture exceed the storage space "
                        + " in the MMU room",
                    "Too many masks Error",
                    JOptionPane.WARNING_MESSAGE);
        } 
         
        catch (SQLException e) {
            ErrorMessages.sqlError(e,null);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NumberFormatException e) {
            JOptionPane.showMessageDialog(
                   null,
                   " ADP File Corrupt \n",
                   "ADP File Corrupt",
                   JOptionPane.WARNING_MESSAGE);
            
        }         
        catch (Exception e) {
            e.printStackTrace();
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
            OutOfMemoryReport.showMessage();
            throw e;
        }
        return "";
    }

    private CalibrationBlock[] getIdenticalMaskObs(CalibrationBlock[] cbList) {
        Vector obVec = new Vector();
        Vector maskSetData = new Vector();
        CalibrationBlock[] obList;
        for (int i = 0; i < cbList.length; i++) {
            MaskNaidInfo naidInfo =
                getNaidInfo(Config.getCfg().uniqueToTableId(cbList[i].getId()));
            boolean found = false;
            for (int j = 0; j < maskSetData.size(); j++) {
                MaskNaidInfo temp = (MaskNaidInfo) maskSetData.elementAt(j);
                if (temp.equals(naidInfo)) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                obVec.addElement(cbList[i]);
                maskSetData.addElement(naidInfo);
            }
        }
        obList = new CalibrationBlock[obVec.size()];
        obVec.copyInto(obList);
        return obList;
    }

    private String[] getNaidList(CalibrationBlock[] cbList)
        throws IOException {
        CalibrationBlock[] newCBList = getIdenticalMaskObs(cbList);
        Instrument inst = null;
        Vector naidList = new Vector();        
        int ts_index = -1;
        StringBuffer naidReport = new StringBuffer();
        if ((newCBList != null) && (newCBList.length > 0)) {
            for (int i = 0; i < newCBList.length; i++) {
                TemplateSignature[] tsfs = null;
                Parameter[] pList = null;
                if (newCBList[i] instanceof ObservationBlock) {
                    pList =
                        ((ObservationBlock) newCBList[i])
                            .getTemplateSignature()
                            .getParamList();
                    ts_index = 0;
                } else {
                    tsfs = newCBList[i].getOd().getSignatures();
                    pList = tsfs[0].getParamList();
                    ts_index = 1;
                }
                String instCode = newCBList[i].getOd().getInstrument();
                inst = InstrumentList.getInstance().getInstrument(instCode);
                if (!inst.hasMasks())
                    // put error message in log here
                    continue;
                for (int j = 0; j < pList.length; j++) {
                    if (pList[j].getName().indexOf("INS.ADP") != -1) {
                        PafFile f =
                            new PafFile(new StringReader(pList[j].getValue()));
                        String id = f.getPafIdentifier();
                        String name = f.getPafName();
                        System.out.println("Naid from files " + id);
                        int keyIndex =
                            getIndexString(pList[j].getParameterName());
                        if (!naidList.contains(id)
                            && !id.equals(((MaskInstrument) inst).getDummyNaid())
                            && getStatus(
                                newCBList[i].getDbaseId(),
                                ts_index,
                                keyIndex).equals(
                                Mask.NO_MASK)
                            || getStatus(
                                newCBList[i].getDbaseId(),
                                ts_index,
                                keyIndex).equals(
                                Mask.MASK_REMANUFACTURE)) {
                            if (isSingleSelect()) {
                                if (inSelectedNaidList(id)) {
                                    System.out.println(
                                        "Adding to selected list " + id);
                                    naidList.addElement(id);
                                    
                                }
                            } else {
                                naidList.addElement(id);
                               
                            }
                        } else
                            System.out.println(
                                "Status = "
                                    + getStatus(
                                        newCBList[i].getDbaseId(),
                                        ts_index,
                                        j)
                                    + " "
                                    + Mask.NO_MASK);
                    }
                }
            }
        }
        //String s = optimiseMaskManufacturing(cbList,obIdVec);
        String[] naidStringList = new String[naidList.size()];
        naidList.copyInto(naidStringList);
        return naidStringList;
    }

    public String getNaidsForMasks(CalibrationBlock[] cbList)
        throws IOException {
        StringBuffer naidReport = new StringBuffer();
        CalibrationBlock[] uniqueList = getIdenticalMaskObs(cbList);
        String report = null;
        try {
            report = optimiseMaskManufacturing(uniqueList, true);
           
        } catch (SQLException e) {
            e.printStackTrace();
        }
        catch(StorageCabinetFullException e) {
                 
             JOptionPane.showMessageDialog(
                    null,
                    " Too many masks to fit in the storage cabinet \n"
                        + "\n Total number of maks stored and inserted and the number "
                        + "\n you wish to manufacture exceed the storage space "
                        + " in the MMU room",
                    "Too many masks Error",
                    JOptionPane.WARNING_MESSAGE);
             return "";
        }       
        return reportString();
    }

    public String reportString() {
        StringBuffer naidReport = new StringBuffer();
        if(naidAdpList_.size() == 0)
             naidReport.append("No Masks will be generared for this order");
        for (int i = 0; i < naidAdpList_.size(); i++)
            naidReport.append(((NaidAdp)naidAdpList_.elementAt(i)).getMaskName() + "\n");
        if (naidAdpList_.size() > 0)
                naidReport.append(
                    "A total of "
                        + naidAdpList_.size()
                        + " will be manufactured for these ob's\n");       
        return naidReport.toString();
    }

    public String getStatus(long ob_id, int ts_index, int adp_index) {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;
        String status = null;
        try {
            conn =
                pool.acquireConnection(
                    Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                    Config.getCfg().getDbUserName(),
                    Config.getCfg().getDbPassword());
            String sql =
                "select  mask_status from "
                    + obrepDbName_
                    + "..masks,"
                    + obrepDbName_
                    + "..obsblk_masks"
                    + " where "
                    + obrepDbName_
                    + "..obsblk_masks.ob_id = "
                    + ob_id
                    + " AND "
                    + obrepDbName_
                    + "..obsblk_masks.ts_index = "
                    + ts_index
                    + " AND "
                    + obrepDbName_
                    + "..obsblk_masks.adp_index = "
                    + adp_index
                    + " AND "
                    + obrepDbName_
                    + "..obsblk_masks.mask_id = "
                    + obrepDbName_
                    + "..masks.mask_id";
            System.out.println("SQL " + sql);
            stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next()) {
                status = rs.getString("mask_status");
            }
            System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!Status  = " + status);
        } catch (SQLException e) {
            e.printStackTrace();
        }
        ConnectionPool.getPool().releaseConnection(conn);
        return status;
    }
    private MaskNaidInfo getNaidInfo(long db_id) {

        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;
        Vector naidVec = new Vector();
        Vector statusVec = new Vector();
        try {
            conn =
                pool.acquireConnection(
                    Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                    Config.getCfg().getDbUserName(),
                    Config.getCfg().getDbPassword());
            stmt = conn.createStatement();
            String sql =
                "SELECT DISTINCT  mask_naid,mask_status from "
                    + obrepDbName_
                    + "..masks, "
                    + obrepDbName_
                    + "..obsblk_masks"
                    + " WHERE "
                    + obrepDbName_
                    + "..obsblk_masks.ob_id = "
                    + db_id
                    + " AND "
                    + obrepDbName_
                    + "..obsblk_masks.mask_id = "
                    + obrepDbName_
                    + "..masks.mask_id";
            System.out.println("SQL " + sql);
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next()) {
                String naid = rs.getString("mask_naid");
                naidVec.addElement(naid);
                String status = rs.getString("mask_status");
                statusVec.addElement(status);
            }
            pool.releaseConnection(conn);

        } catch (SQLException e) {
        } finally {
            pool.releaseConnection(conn);
        }
        String[] naidArray = new String[naidVec.size()];
        naidVec.copyInto(naidArray);
        return new MaskNaidInfo(naidArray);
    }

    private String optimiseMaskManufacturing(
        CalibrationBlock[] cbList,
        boolean report)
        throws IOException, SQLException,StorageCabinetFullException,NumberFormatException {
        StringBuffer paf = new StringBuffer();
        MaskInstrument inst = null;
        int ts_index = -1;
        naidAdpList_.removeAllElements();       
        cannotMan_.removeAllElements();
        boolean processheader = false;
        Vector toParameterFormat = new Vector();
        Vector tsVec = new Vector();
        int totalCount = MaskDBIO.getMaskStoredCount();
        if ((cbList != null) && (cbList.length > 0)) {
            Vector naidList = new Vector();
            for (int i = 0; i < cbList.length; i++) {
                boolean alldumies = true;
                TemplateSignature ts1 = null;
                TemplateSignature[] tsfs = null;
                Parameter[] pList = null;
                if (cbList[i] instanceof ObservationBlock) {
                    ts1 =
                        new TemplateSignature(
                            ((ObservationBlock) cbList[i])
                                .getTemplateSignature());
                    pList =
                        ((ObservationBlock) cbList[i])
                            .getTemplateSignature()
                            .getParamList();
                    ts_index = 0;
                } else {
                    tsfs = cbList[i].getOd().getSignatures();
                    ts1 = new TemplateSignature(tsfs[0]);
                    pList = tsfs[0].getParamList();
                    ts_index = 1;
                }
                tsVec.addElement(ts1);
                String instCode = cbList[i].getOd().getInstrument();
                inst = (MaskInstrument)InstrumentList.getInstance().getInstrument(instCode);
                if (!inst.hasMasks())
                    // put error message in log here
                    continue;
                for (int j = 0; j < pList.length; j++) {
                    if (pList[j].getName().indexOf("INS.ADP") != -1) {
                        PafFile f =
                            new PafFile(new StringReader(pList[j].getValue()));
                        String id = f.getPafIdentifier();                       
                        int keyIndex =
                            getIndexString(pList[j].getParameterName());
                        if (!naidList.contains(id)
                            && !id.equals(((MaskInstrument) inst).getDummyNaid())
                            && getStatus(
                                cbList[i].getDbaseId(),
                                ts_index,
                                keyIndex).equals(
                                Mask.NO_MASK)
                            || getStatus(
                                cbList[i].getDbaseId(),
                                ts_index,
                                keyIndex).equals(
                                Mask.MASK_REMANUFACTURE)) {
                            if (isSingleSelect()) {
                                if (inSelectedNaidList(id)) {
                                    naidList.addElement(id);
                                    alldumies = false;
                                    MaskQueueManager mgr = new MaskQueueManager();
                                    naidAdpList_.addElement(
                                        mgr.new NaidAdp(
                                            id,
                                            f,
                                            keyIndex,
                                            cbList[i].getId(),
                                            pList));
                                    totalCount++;
                                   
                                } else {
                                    //cannotMan_.addElement(id);
                                    pList[j].setValue(
                                        TextUtils.revertEscapedString(
                                            ((MaskInstrument) inst)
                                                .getDummyAdm()));
                                    
                                }
                            } else {
                                naidList.addElement(id);
                                alldumies = false;
                                MaskQueueManager mgr = new MaskQueueManager();

                                naidAdpList_.addElement(
                                    mgr.new NaidAdp(
                                        id,
                                        f,
                                        keyIndex,
                                        cbList[i].getId(),
                                        pList));
                                totalCount++;
                            }
                        } else {
                            if (isSingleSelect()) {
                                if (inSelectedNaidList(id))
                                    cannotMan_.addElement(id);
                            } else
                                cannotMan_.addElement(id);
                            pList[j].setValue(
                                TextUtils.revertEscapedString(
                                    ((MaskInstrument) inst).getDummyAdm()));
                        }
                    }
                }
                /*if(!report || debug_) {
                    if(!alldumies) {
                        if(!processheader) {
                           toParameterFormat.addElement(cbLisi[i])
                           tsVec.addElement(ts1) 
                           paf.append(cbList[i].toOBDFormat(true));
                           processheader=true;
                        }
                        else {
                           toParameterFormat.addElement(cbLisi[i])
                           tsVec.addElement(ts1); 
                           paf.append(cbList[i].toOBDFormat(false));
                        }
                    }
                    }*/
                //if(cbList[i] instanceof ObservationBlock)
                //    ((ObservationBlock)cbList[i]).setTemplateSignature(ts1);
                //else if(cbList[i] instanceof CalibrationBlock)
                //    tsfs[0] = ts1;    
            }
            System.out.println("Total Count " + totalCount);
            System.out.println("Total storage Count " + 100*inst.getCabinetCount());           
            if (totalCount > (100*inst.getCabinetCount())) {
                // XXX Warn user that mask limit exceeded.
                throw new StorageCabinetFullException("Stoarage Cabinet Full");                           
            }
            if (!report || debug_) {
                if (inst != null
                    && inst.hasMasks()
                    && naidAdpList_.size() > 0) {
                    MaskQueueManager.setMaskStatusFromNaid(
                        naidAdpList_,
                        Mask.PENDING_MANUFACTURED,
                        ((MaskInstrument) inst).getCabinetCount());
                }
            }
            
            if (!report || debug_) {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                for (int i = 0; i < cbList.length; i++) {
                    if (i == 0)
                        paf.append(cbList[i].toOBD(true,objMgr.getDbaseId(cbList[i]),OHSApplication.getShortDesc()));
                    else
                        paf.append(cbList[i].toOBD(false,objMgr.getDbaseId(cbList[i]),OHSApplication.getShortDesc()));
                }
            }
            for (int i = 0; i < cbList.length; i++) {
                if (cbList[i] instanceof ObservationBlock)
                    ((ObservationBlock) cbList[i]).setTemplateSignature(
                        (TemplateSignature) tsVec.elementAt(i));
                else if (cbList[i] instanceof CalibrationBlock) {
                    TemplateSignature[] tsfs =
                        cbList[i].getOd().getSignatures();
                    tsfs[0] = (TemplateSignature) tsVec.elementAt(i);
                }
            }

        }
        String pafText = paf.toString();
        System.out.println("PAF STRING = " + paf.toString());
        if (!report) {
            saveFile(TextUtils.currentISODateTime() + ".mmo",pafText);            
            return pafText;
        }
        return "report";
    }

    public void receiveReport(String report) {
        try {
            //Updater.propagateMaskInfo();
            MaskInstrument inst =
                (MaskInstrument) InstrumentList.getInstance().getInstrument(
                    "VIMOS");
            PafFile paf = new PafFile(new StringReader(report));
            PafData[] pd = paf.getIdentifierList();
            StringBuffer sb = new StringBuffer();
            for (int j = 0; j < pd.length; j++) {
                String name = pd[j].getIdentifier();
                System.out.println("Name = " + name);
                if (name.startsWith("INS.MASK") && name.endsWith(".ID")) {

                    sb.append("'" + pd[j].getValue() + "'");
                    if (j < pd.length - 1)
                        sb.append(',');
                }

            }
            if (sb.length() == 0)
                return;

            String maskIdList = sb.toString();
            String sql =
                "UPDATE "
                    + obrepDbName_
                    + "..masks set "
                    + obrepDbName_
                    + "..masks.mask_status = "
                    + SqlUtils.sqlEscapedString(Mask.MASK_MANUFACTURED)
                    + " WHERE "
                    + obrepDbName_
                    + "..masks.bar_code in ("
                    + maskIdList
                    + ")";
            executeUpdate(sql);
            MaskQueueManager.fireMaskPropertyChange(
                new MaskEvent("MaskStatus", Mask.MASK_MANUFACTURED));
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private void executeUpdate(String sql) throws SQLException {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;

        try {
            conn =
                pool.acquireConnection(
                    Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                    Config.getCfg().getDbUserName(),
                    Config.getCfg().getDbPassword());

            stmt = conn.createStatement();

            System.out.println("sql = " + sql);
            stmt.executeUpdate(sql);
        } finally {
            if (conn != null) {
                try {
                    stmt.close();
                } catch (SQLException e) {
                    // ignore
                }
                pool.releaseConnection(conn);
            }
        }
    }

    private String findKeywordValue(String id) {
        for (int i = 0; i < identifierList_.length; i++) {
            if (identifierList_[i].getIdentifier().equals(id))
                return identifierList_[i].getValue();
        }
        return null;
    }

    public class MaskNaidInfo {

        public String[] maskNaids_ = {
        };

        //public String [] maskStatuses_ = {};

        public MaskNaidInfo(String[] maskNaids) {
            maskNaids_ = maskNaids;
        }

        public String toString() {
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < maskNaids_.length; i++) {
                buf.append("NAID = " + maskNaids_);
            }
            return buf.toString();
        }

        public boolean equals(MaskNaidInfo maskNaidInfo) {
            String[] maskNaids = maskNaidInfo.maskNaids_;
            if (maskNaids_.length != maskNaids.length)
                return false;
            for (int i = 0; i < maskNaids.length; i++) {
                if (!maskNaids[i].equals(maskNaids_[i]))
                    return false;
            }
            return true;
        }
    }


  private  class StorageCabinetFullException extends Exception {

    /**
     * Construct an exception without explanatory message (not recommended)
     */
    public StorageCabinetFullException() {
	super();
    }

    /**
     * Construct an exception with accompanying explanatory text.
     * 
     * @param msg   message giving a reason for the exception.
     */
    public StorageCabinetFullException(String msg) {
	super(msg);
    }

  }
}
