package org.eso.ohs.phase2.apps.ot.wizard;

import java.text.ParseException;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DBdataHelper;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;
import org.eso.ohs.phase2.apps.ot.Persistence;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.orang.OBInfoLoader;
import org.eso.ohs.phase2.orang.OBRankingEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.orang.ObWeatherFilter;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.orang.OrangScriptingBox;
import org.eso.ohs.phase2.timeline.EarlyGreedy;
import org.eso.ohs.phase2.timeline.Timeline;
import org.eso.ohs.phase2.timeline.TimelineView;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;
import org.eso.ohs.phase2.visibility.VisicalcOptions;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * This class implements all functionality needed to run the ORANG, either via
 * the wizard or another interface.
 * 
 * Methods of this class take all info about the current state of the OB ranking
 * Engine from an {@link org.eso.ohs.phase2.apps.ot.wizard.OrangModel}
 * 
 * @version $Revision: 1.30 $
 * @author pnunes, ddorigo
 */

// $Id: OrangEngine.java,v 1.30 2005/12/12 16:29:53 ddorigo Exp $
public class OrangEngine implements OBListener {
    private static final String OB_STATUS_WRONG = "NoStatus";

    private Vector obList = new Vector();

    /**
     * Logger
     */
    protected static Logger stdlog_ = Logger.getLogger(OrangEngine.class);

    private ObWeatherFilter obWeatherFilter = null;

    private Vector obVisicalcInputList = new Vector();

    private Vector obVisicalcOutList = new Vector();

    private Vector obRanked = new Vector();

    private Queue queueObservable = null;

    private Queue queueNotObservable = null;

    /** Widget to display the current timeline */
    private TimelineView timelineView = null;

    /** Top-level window to display the current timeline */
    private JFrame timelineFrame;

    /**
     * initialise the objet member.
     */
    public OrangEngine() {
        init();
    }

    /**
     * reinitialize the engine
     */
    private void init() {
    	Phase1SelectStmt.definePhase1TableDb("opc70", "dbo");
        Phase2SelectStmt.definePhase2TableDb("obrep2", "dbo");
        setObList(new ObDataContainer[0]);
        setObWeatherFilter(new ObWeatherFilter(new ObDataContainer[0]));
        setObVisicalcInputList(new ObDataContainer[0]);
        setObVisicalcOutList(new OBScheduleInfo[0]);
        setObRanked(new OBScheduleInfo[0]);
    }

    public void loadObList(OrangModel model)
            throws ObjectNotFoundException, ObjectIOException {
        OBInfoLoader obLoader = new OBInfoLoader();
        int queueSelectionType = model.getObSelectionType();
        if (queueSelectionType == OrangModel.OB_SELECTION_INSTRUMENT) {
            String[] queueList = model.getQueuesInstruments();

            setObList(obLoader.loadQueues(queueList, model.getUser(), model
                    .isSuperUser()));
        } else if (queueSelectionType == OrangModel.OB_SELECTION_TELESCOPE) {
            String[] queueList = model.getQueuesTelescopes();

            setObList(obLoader.loadQueues(queueList, model.getUser(), model
                    .isSuperUser()));
        } else if (queueSelectionType == OrangModel.OB_SELECTION_QUEUES) {
            setObList(obLoader.loadQueues(model.getSelectedQueues(), model
                    .getUser(), model.isSuperUser()));
        } else {
            throw new UnsupportedOperationException("operation "
                    + queueSelectionType + " in selection not supported");
        }
    }

    /**
     * @param model
     *            the weathe condition passed by the model
     */
    public void filterObByWeather(OrangModel model) {
        setObWeatherFilter(new ObWeatherFilter(getObList()));
        setObVisicalcInputList(getObWeatherFilter().filterObArray(
                model.getWeatherConstraint()));
    }

    public void filterObByConstraints(OrangModel model)
            throws ObjectNotFoundException, ObjectIOException,
            ValueRejectedException {
        VisibilityJCalc visibilityCalc = new VisibilityJCalc();
        VisicalcOptions visiCalc = model.getVisiCalcOptions();

        setObVisicalcOutList(visibilityCalc.calc(getObVisicalcInputList(),
                model.getStepInterval(), model.getStartDate(), model
                        .getEndDate(), visiCalc, model.getLatitude(), model
                        .getLongitude()));

    }

    /**
     * Compute and show a timeline from the current list of ranked OBS.
     * 
     * @param model
     *            We take the current ORANG settings form here.
     */
    // amchavan & pnunes: added, 17-Oct-2005
    public void showTimeline(OrangModel model) {
        // Compute a timeline using the early-greedy algorithm
        Timeline timeline = new EarlyGreedy(getObRanked(),
                model.getStartDate(), model.getEndDate());
        System.out.print(timeline);

        if (timelineView == null) {
            timelineView = new TimelineView(timeline);
            timelineFrame = new JFrame("Early-greedy timeline");
            timelineFrame.setContentPane(timelineView);
            timelineFrame.pack();
        } else {
            timelineView.setTimeline(timeline);
        }
        timelineFrame.setVisible(true);
    }

    public void applyAlgorithm(OrangModel model) throws OrangScriptException {
        String algorithm = model.getRankingAlgorithm();
        OBRankingEngine engine = OrangScriptingBox.getInstance();
        engine.setOBsForRanking(getObservableOBs());
        engine.rank(algorithm);
        setObRanked(engine.getRankedOBs());
    }

    public void showRankedObs(OrangModel model) {
        try {

            //  

            // initialise observable queue
          //  Vector queueItemObservableVector = new Vector();
            // initialise not observable queue
          //  Vector queueItemNotObservableVector = new Vector();

            String orangQueueName = OHSConfig.getInstance().getValue(
                    "ORANG.OBSERVABLE.NAME");
            String orangNotObsQueueName = OHSConfig.getInstance().getValue(
                    "ORANG.NOTOBSERVABLE.NAME");
            Queue queueObservableOld = queueObservable;
            Queue queueNotObservableOld = queueNotObservable;
            queueObservable = createNewQueue(model, orangQueueName);
            queueNotObservable = createNewQueue(model, orangNotObsQueueName);

            OBScheduleInfo[] obNotObservable = getNonObservableOBs();
           QueueItem[] notObsQueueItems= buildNotObservQueueItems(obNotObservable,
                    queueNotObservable);
         //  saveQueueItems(queueNotObservable, notObsQueueItems);
           
           OBScheduleInfo[] obObservable = getObservableOBs();
           QueueItem[] obsQueueItems= buildObservQueueItems(obObservable,
                   queueObservable);          
          //  saveQueueItems(queueObservable, obsQueueItems);
            
            VisiPlotOptions settings = generateSettings(model);

            OTViewManager otViewMgr = OTViewManager.getOTViewMgr();
            otViewMgr.forceQueueClose(queueObservableOld);
            otViewMgr.forceQueueClose(queueNotObservableOld);
            otViewMgr.showOrangNotObservableQueue(queueNotObservable, settings,model,
                    obNotObservable,notObsQueueItems);
            otViewMgr.showOrangObservableQueue(queueObservable, settings,model,
                    obObservable,obsQueueItems);

        } catch (ObjectIOException e) {
            JOptionPane.showMessageDialog(null,
                    "An error occured while registering the queue", "Error",
                    JOptionPane.WARNING_MESSAGE);
            e.printStackTrace();
        }
    }

    /**
     * @param model
     * @param otViewMgr
     * @param orangQueueName
     * @param queueObservable2
     * @throws ObjectIOException
     *             void
     */
    private Queue createNewQueue(OrangModel model, String orangQueueName)
            throws ObjectIOException {
        Queue currentQueue = saveQueue(orangQueueName, model.getStartISODate(),
                model.getEndISODate());
        return currentQueue;
    }

    /**
     * @param model
     * @return VisiPlotOptions
     */
    private VisiPlotOptions generateSettings(OrangModel model) {
        VisiPlotOptions settings = new VisiPlotOptions();

        settings.setValue(VisiPlotOptions.dataQueryIntKey, model
                .getStepInterval());
        settings
                .setValue(VisiPlotOptions.startTimeKey, model.getStartISODate());
        settings.setValue(VisiPlotOptions.endTimeKey, model.getEndISODate());
        settings.setValue(VisiPlotOptions.fliConEnabledKey, model
                .isFliConstrain());
        settings.setValue(VisiPlotOptions.moonDistConEnabledKey, model
                .isMoonDistanceConstrain());
        settings.setValue(VisiPlotOptions.sunConEnabledKey, model
                .isSunAngleConstrain());
        settings.setValue(VisiPlotOptions.obtimeIntervalConKey, model
                .isTimeIntervalConstrain());
        settings.setValue(VisiPlotOptions.obSiderealTimeIntervalConKey, model
                .isSidTimeIntervalConstrain());
        settings.setValue(VisiPlotOptions.sunConKey, model.getSunAngleDegree());
        settings.setValue(VisiPlotOptions.zenithConKey, model.isTelescopeZenithConstraint());
        
        return settings;
    }

    /**
     * @param queueItemObservableVector
     * @param queueItemNotObservableVector
     * @param queueObservable
     * @param queueNotObservable
     *            void
     */
    private QueueItem[] buildNotObservQueueItems(
            OBScheduleInfo[] queueItemNotObservable,
            Queue queueNotObservable) {
        QueueItem[] queueItemResult = new QueueItem [queueItemNotObservable.length];
        for (int i = 0; i < queueItemNotObservable.length; i++) {
            OBScheduleInfo currentOb =queueItemNotObservable[i];
                QueueItem item = new QueueItem(currentOb
                        .getOb()
                        .getObInfo()
                        .getId(), queueNotObservable.getId(), i,
                        ObservationBlock.OBSBLK_TYPE);             
                queueItemResult[i] = item;
            }
         return queueItemResult;
    }
    /**
     * @param queueItemObservableVector
     * @param queueItemNotObservableVector
     * @param queueObservable
     * @param queueNotObservable
     *            void
     */
    private QueueItem[] buildObservQueueItems(
            OBScheduleInfo[] queueItemObservable,
            Queue queueObservable) {
        QueueItem[] queueItemResult = new QueueItem [queueItemObservable.length];
         
        for (int i = 0; i < queueItemObservable.length; i++) {
            OBScheduleInfo currentOb = queueItemObservable[i];
                QueueItem item = new QueueItem(currentOb
                        .getOb()
                        .getObInfo()
                        .getId(), queueObservable.getId(), i,
                        ObservationBlock.OBSBLK_TYPE, currentOb
                                .getRankingScore());
                
                queueItemResult[i] = item;
           
        }
        return queueItemResult;
    }

    /**
     * @param date2
     * @param date
     * @param objMgr
     * @return
     * @throws ObjectIOException
     *             Queue
     */
    private Queue saveQueue(String name, String startISODate, String endISODate)
            throws ObjectIOException {
        Persistence objMgr = Persistence.getInstance();
        Queue queue = (Queue) objMgr.createAndRegister(Media.DBASE, objMgr
                .getRoot(Media.DBASE), Queue.class);
        try {
            long startLong = Convert.ISOdateToLong(startISODate);
            long endLong = Convert.ISOdateToLong(endISODate);

            queue.setIsNew(true);
            queue.setDuration(endLong - startLong);
            queue.setStartTime(startLong);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        queue.setName(name);
        queue.setTelescope("UT1");
        return queue;
    }

    /**
     * @param objMgr
     * @param queue
     * @param queueItemArray
     * @throws ObjectNotFoundException
     * @throws ObjectIOException
     *             void
     */
    private void saveQueueItems(Queue queue,
            QueueItem[] queueItemArray)
            throws ObjectNotFoundException, ObjectIOException {
        Persistence objMgr = Persistence.getInstance();
      
        queue.setQueueList(queueItemArray);
       // objMgr.putBusObj(queue);
    }

    public void clear() {
        init();
    }

    public String getLoadedInfo() {

        String res = "0 Observation Blocks loaded";

        if (getObList() != null) {
            res = getObList().length + " Observation Blocks loaded";
        }

        return (res);

    }

    public String getWeatherInfo() {
        String res = "0 Observation Blocks considered";

        res = getObVisicalcInputList().length
                + " Observation Blocks considered. "
                + (getObList().length - getObVisicalcInputList().length)
                + " filtered out.";
        return (res);
    }

    public String getVisibilityInfo() {
        String res = "0 Observation Blocks considered";

        int observable = 0;
        int not_observable = 0;
        OBScheduleInfo[] currentVisicalc = getObVisicalcOutList();
        for (int i = 0; i < currentVisicalc.length; i++) {
            OBScheduleInfo ob = currentVisicalc[i];
            if (ob.isObservable()) {
                observable++;
            } else {
                not_observable++;
            }

        }

        res = observable + " Observation Blocks considered. " + not_observable
                + " filtered out.";

        return (res);
    }

    public String getRankedInfo() {
        String res = "0 Observation Blocks ranked";

        int observable = 0;
        OBScheduleInfo[] currentVisicalc = getObVisicalcOutList();
        for (int i = 0; i < currentVisicalc.length; i++) {
             OBScheduleInfo ob = currentVisicalc[i];
            if (ob.isObservable()) {
                observable++;
            }

        }

        res = observable + " Observation Blocks ranked. ";

        return (res);
    }

    /**
     * @param obList
     *            The obList to set.
     */
    private void setObList(ObDataContainer[] obList) {
        this.obList.removeAllElements();
        CollectionUtils.addAll(this.obList, obList);
    }

    /**
     * @return Returns the obList.
     */
    private ObDataContainer[] getObList() {
        return (ObDataContainer[]) obList.toArray(new ObDataContainer[obList
                .size()]);
    }

    /**
     * @param obWeatherFilter
     *            The obWeatherFilter to set.
     */
    private void setObWeatherFilter(ObWeatherFilter obWeatherFilter) {

        this.obWeatherFilter = obWeatherFilter;
    }

    /**
     * @return Returns the obWeatherFilter.
     */
    private ObWeatherFilter getObWeatherFilter() {
        return obWeatherFilter;
    }

    /**
     * @param obVisicalcInputList
     *            The obVisicalcInputList to set.
     */
    private void setObVisicalcInputList(ObDataContainer[] obVisicalcInputList) {
        this.obVisicalcInputList.removeAllElements();
        CollectionUtils.addAll(this.obVisicalcInputList, obVisicalcInputList);
    }

    /**
     * @return Returns the obVisicalcInputList.
     */
    private ObDataContainer[] getObVisicalcInputList() {
        return (ObDataContainer[]) obVisicalcInputList
                .toArray(new ObDataContainer[obVisicalcInputList.size()]);
    }

    /**
     * @param obVisicalcOutList
     *            The obVisicalcOutList to set.
     */
    private void setObVisicalcOutList(OBScheduleInfo[] obVisicalcOutList) {
        this.obVisicalcOutList.removeAllElements();
        CollectionUtils.addAll(this.obVisicalcOutList, obVisicalcOutList);
    }

    /**
     * @return Returns the obVisicalcOutList.
     */
    private OBScheduleInfo[] getObVisicalcOutList() {
        return (OBScheduleInfo[]) obVisicalcOutList
                .toArray(new OBScheduleInfo[obVisicalcOutList.size()]);
    }

    /**
     * @param obRanked
     *            The obRanked to set.
     */
    private void setObRanked(OBScheduleInfo[] obRanked) {
        this.obRanked.removeAllElements();
        CollectionUtils.addAll(this.obRanked, obRanked);
    }

    /**
     * @return Returns the obRanked.
     */
    private OBScheduleInfo[] getObRanked() {
        return (OBScheduleInfo[]) obRanked.toArray(new OBScheduleInfo[obRanked
                .size()]);
    }

    /**
     * Returns an array of observable OBs
     * 
     * @return the observable obs
     */
    private OBScheduleInfo[] getObservableOBs() {
        OBScheduleInfo[] obs=getObVisicalcOutList();
        LinkedList observable = new LinkedList();

        for (int i = 0; i < obs.length; i++) {
            OBScheduleInfo ob = obs[i];
            if (ob.isObservable()) {
                observable.add(ob);
            }
        }
        OBScheduleInfo[] res = (OBScheduleInfo[]) observable
                .toArray(new OBScheduleInfo[observable.size()]);
        return (res);

    }

    /**
     * Returns an array of non-observable OBs from the first list of obs
     * 
   
     * @return the list of not observable
     */
    private OBScheduleInfo[] getNonObservableOBs() {
        ObDataContainer[] allObs = getObList();
        LinkedList notObservable = new LinkedList();

        for (int i = 0; i < allObs.length; i++) {
            OBScheduleInfo ob = new OBScheduleInfo();
            ob.setOb(allObs[i]);
            int currentObIndex = obVisicalcOutList.indexOf(ob);
            if (currentObIndex < 0) { // element not in visicalc means not observable
                notObservable.add(ob);
            } else { // must verify element is not observable after visibility calc
               OBScheduleInfo visicalcOb = (OBScheduleInfo) obVisicalcOutList.get(currentObIndex);
               if (!visicalcOb.isObservable()) {
                   notObservable.add(ob);
               }
            }
        }
        OBScheduleInfo[] res = (OBScheduleInfo[]) notObservable
                .toArray(new OBScheduleInfo[notObservable.size()]);
        return (res);

    }

    /**
     * if a status change event and the new status of the ob is executed or
     * completed, it removes the ob from all the orang lists
     * 
     * @see org.eso.ohs.persistence.OBListener#obPropertyChange(org.eso.ohs.persistence.OBEvent)
     */
    public synchronized void obPropertyChange(OBEvent event) {
        // if delete event don't do anything. Check only when is a status change
        // event
        if (!event.getValue().equals(DELETE_EVENT)) {
            long[] OBIDs = event.getOBsId();
            for (int i = 0; i < OBIDs.length; i++) {

                String status = event.getValue();   
                long dbId = Config.getCfg().uniqueToTableId(OBIDs[i]);
                if (status != null
                        && !status.equals(OB_STATUS_WRONG)
                        && (status.equals(ObservationBlock.EXECUTED) || status
                                .equals(ObservationBlock.COMPLETED))) {
                    ObTargetConstraintInfo currentOb = new ObTargetConstraintInfo();
                    currentOb.setDbaseId(dbId);
                    ObDataContainer currentObContainer = new ObDataContainer(
                            currentOb);
                    int objPosition = obList.indexOf(currentObContainer);
                    if (objPosition >= 0) {
                        obList.remove(objPosition);
                    }

                    objPosition = obVisicalcInputList
                            .indexOf(currentObContainer);
                    if (objPosition >= 0) {
                        obVisicalcInputList.remove(objPosition);
                    }

                    OBScheduleInfo currentSchedOb = new OBScheduleInfo();
                    currentSchedOb.setOb(currentObContainer);
                    objPosition = obVisicalcOutList.indexOf(currentSchedOb);
                    if (objPosition >= 0) {
                        obVisicalcOutList.remove(objPosition);
                    }
                    objPosition = obRanked.indexOf(currentSchedOb);
                    if (objPosition >= 0) {
                        obRanked.remove(objPosition);
                    }
                }
            }
        }
    }
}
