package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.EventObject;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolTip;
import javax.swing.ScrollPaneConstants;
import javax.swing.plaf.basic.BasicComboBoxEditor;
import javax.swing.text.PlainDocument;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.gui.models.DeclinationModel;
import org.eso.ohs.core.gui.models.FloatListModel;
import org.eso.ohs.core.gui.models.FloatModel;
import org.eso.ohs.core.gui.models.InputMaskModel;
import org.eso.ohs.core.gui.models.IntListModel;
import org.eso.ohs.core.gui.models.IntModel;
import org.eso.ohs.core.gui.models.RightAscensionModel;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.FilePreviewDialog;
import org.eso.ohs.core.gui.widgets.MultiLineToolTip;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.WildCardExtensionFileFilter;
import org.eso.ohs.core.utilities.CRC32PAFFile;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.Paramfile;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.docviewdatatrans.DegreeAdaptor;
import org.eso.ohs.instruments.CoordParameter;
import org.eso.ohs.instruments.KeywordListParameter;
import org.eso.ohs.instruments.KeywordParameter;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;

/**
 * This class renders the appropiate editors for the editing of parametets.
 * When the user types or click on the cell selected in the grid. The
 * appropiate editor is selected.
 *
 * $Id: EditorFactory.java,v 1.23 2005/02/11 11:00:29 tcanavan Exp $
 * @version $Revision $
 */
public class EditorFactory extends DefaultCellEditor {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(EditorFactory.class);

    /** The component used for editing within a cell */
    private Component editorComponent_;

    /** The component used when a popup dialog happens */
    private static P2PPFileChooser p2ppFileChooser_;

    private KeywordListDialog keywordListDialog_;

    /** Float entry mask */
    private FloatListModel floatListModel_;

    /** Integer list mask */
    private IntListModel intListModel_;

    /** Float list mask */
    private FloatModel floatModel_;

    /** Integer mask */
    private IntModel intModel_;

    /** Declination model */
    private DeclinationModel declinationModel_;

    /** Right Ascension Model */
    private RightAscensionModel rightAscensionModel_;

    /** Data stored in Combination Box */
    private Hashtable comboData_;

    /** Current Parameter being editied */
    private Parameter currParam_;

    /** Reference to the table displaying the parameter */
    private ODTemplateTable paramTable_;

    /** Row being editied */
    private int row_;

    /** Column being editied */
    private int column_;

    /** Tool tip to be displayed in textfield */
    private String tooltp_;

    /** Key listener to be added to editorComponent_ */
    private KeyListener keyListener_;

    /** Reference to a multiline tooltip control */
    private static MultiLineToolTip multiLineToolTip_ = new MultiLineToolTip();

    private ComboKeyListener comboKeyList_ = new ComboKeyListener();

    private static P2PPFilePreviewDialog filePreviewDialog_;

    private static SwingFileChooser fileSaveDialog_;

    private boolean shiftPressed_ = false;

    private boolean parameterFileDirExists_ = false;


    /**
     * Intitialise the factory and all of the input masks
     */
    EditorFactory(JTextField textField) {

        super(textField);

        String paramDir;
        File paramDirFile;

        AppConfig appcfg = AppConfig.getAppConfig();
        paramDir = appcfg.getParameterDir();
        paramDirFile = new File(paramDir);
        stdlog_.debug("CLASS EditorFactory - EditorFactory: paramDir = " + paramDir);
        if (paramDirFile.exists())
            parameterFileDirExists_ = true;

        editorComponent_ = new P2PPTextField("");
        floatListModel_ = new FloatListModel();
        floatModel_ = new FloatModel();
        intListModel_ = new IntListModel();
        intModel_ = new IntModel();
        rightAscensionModel_ = new RightAscensionModel();
        declinationModel_ = new DeclinationModel();
        filePreviewDialog_ = new P2PPFilePreviewDialog(null, "Select File");
        fileSaveDialog_ = new SwingFileChooser();
        filePreviewDialog_.P2PPsetWorkingDirectory(paramDirFile);
        fileSaveDialog_.setLocation(100, 100);
        filePreviewDialog_.setLocation(100, 100);
        setClickCountToStart(1);
        initKeyListener();
    }


    /**
     * Return new value from cell
     * @return Object New Value of cell
     */
    public Object getCellEditorValue() {
        if (editorComponent_ instanceof P2PPComboBox) {
            if (((P2PPComboBox)editorComponent_).isEditable()) {
                P2PPTextField edit =
                    (P2PPTextField) ((P2PPComboBox)editorComponent_)
                        .getEditor()
                        .getEditorComponent();
                if (edit != null) {
                    String s = edit.getText();
                    ((P2PPComboBox)editorComponent_).addItem((Object)s);
                    ((P2PPComboBox)editorComponent_).setSelectedItem((Object)s);
                    ((KeywordParameter)currParam_).setValue(s);
                    ((KeywordParameter)currParam_).setKeywordList();
                    comboData_.put(s, s);
                    edit.removeKeyListener(comboKeyList_);
                }
            }

            paramTable_.requestFocus();
        }
        return ((P2PPComponent)editorComponent_).getValue();
        //return currParam_.getValue();
    }

    public boolean stopCellEditing() {
        boolean stop = super.stopCellEditing();
        if (column_ > -1) {
            paramTable_.setRowSelectionInterval(row_, row_);
            paramTable_.setColumnSelectionInterval(column_, column_);
        }
        return stop;
    }


    /**
     * Called when a select cell event occurs
     * @param anEvent select cell event
     * @return boolean
     */
    public boolean shouldSelectCell(EventObject anEvent) {
        String type = currParam_.getType();
        if (isCellEditable(anEvent)) {
            paramTable_.setRowSelectionInterval(row_, row_);
            paramTable_.setColumnSelectionInterval(column_, column_);
            if (type.equals("keyword")) {
                ((P2PPComboBox)editorComponent_).setPopupVisible(true);
                ((P2PPComboBox)editorComponent_).showPopup();
                return true;
            }

            if (type.equals("keywordlist")) {
                paramTable_.stopCellEditing();
            }
            if (type.equals("file")
                || type.equals("filename")
                || type.equals("paramfile")) {
                paramTable_.stopCellEditing();
            }
            if (type.equals("boolean")) {
                ((P2PPCheckBox)editorComponent_).setValue(currParam_.getValue());
            }

        }
        return true;
    }


    public void setShiftPressed(boolean shiftPressed) {
        shiftPressed_ = shiftPressed;
    }


    /**
     * Save Param file to disk
     * @param paramValue Value to be saved
     * @param f File handle to save value to
     */
    private void saveParamFile(String paramValue, File f) {
        File parent = new File(f.getParent());
        if (!parent.canWrite()) {
            JOptionPane.showMessageDialog(
                paramTable_,
                "No write permissions to save file to directory " + parent.getName(),
                "No write permissions",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        if (f.exists()) {
            int response =
                JOptionPane.showConfirmDialog(
                    paramTable_,
                    "Do you wish to overwrite " + f.getName() + " ?");
            if (response != JOptionPane.YES_OPTION) {
                return;
            }
        }
        try {
            FileIOUtils.stringToFile(paramValue, f.getAbsolutePath());
        } catch (IOException e) {
            ErrorMessages.announceIOError(paramTable_, e);
        }
    }


    /**
     * Set the file preview extension
     * @param fileRange
     */
    private void setFilePreviewExtension(String fileRange) {
        if (fileRange != null && fileRange.length() > 0) {
            int index = fileRange.indexOf('*');
            if (index != -1) {
                String extension = fileRange.substring(index + 1);
                WildCardExtensionFileFilter fileFilter =
                    new WildCardExtensionFileFilter(extension, null);
                filePreviewDialog_.setFileFilter(fileFilter);
                filePreviewDialog_.removeFileFilters();
            }
        } else {
            filePreviewDialog_.setFileFilter(
                filePreviewDialog_.getAcceptAllFileFilter());
            filePreviewDialog_.removeFileFilters();

        }
    }


    /**
     * Return the component that should be used for editing
     * @param table JTable reference
     * @param value value of cell
     * @param unused  Unused parameter.
     * @param row row index
     * @param column column index
     * @return Component
     *
     * <br>ChangeLog: AMC, 04-Jul-2001: fixed 1452
     * <br>ChangeLog: AMC, 10-Aug-2001: Refactored
     */
    public Component getTableCellEditorComponent(
        JTable table,
        Object value,
        boolean unused,
        int row,
        int column) {

        TemplateModel model = (TemplateModel)table.getModel();
        Object obj = model.getDataItem(column);

        paramTable_ = (ODTemplateTable)table;
        row_ = row;
        column_ = column;

        // IF   we are not on a TS column
        // THEN we just highlight and return the current cell
        if (!(obj instanceof TemplateSignature))
            return highlightComponent();

        TemplateSignature ts = (TemplateSignature)obj;
        Parameter[] plist = ts.getDisplayParamList();
        if (plist == null)
            // IF   the template has no (displayed) parameters
            // THEN we just highlight and return the current cell
            return highlightComponent();

        int rowCount = plist.length;
        if (row >= rowCount)
            // IF   user clicked below the last parameter cell
            // THEN we just highlight and return the current cell
            return highlightComponent();

        currParam_ = plist[row];
        tooltp_ = currParam_.getToolTipText();

        String type = currParam_.getType();
        String miniHelp = currParam_.getMiniHelp();

        if (miniHelp != null && miniHelp.length() > 0)
            tooltp_ = "Range : " + tooltp_ + "\n" + "Help : " + miniHelp;

        // Build an editor based on the parameter type and
        // highlight it
        return editorComponent(type, (String)value, tooltp_);
    }


    /**
     * Build and return an editor component based on a parameter's
     * type and value
     *
     * <br>ChangeLog: A M Chavan, 08-Aug-2001
     */
    private Component editorComponent(String type, String value, String tooltp) {

        if (type.equals("boolean"))
            return booleanEditor(value);

        if (type.equals("keywordlist")) {

            // window is the containing window
            Frame window = (Frame)paramTable_.getTopLevelAncestor();
            return keywordlistEditor(value, window);
        }

        if (type.equals("filename"))
            return filenameEditor(value);

        if (type.equals("file"))
            return fileEditor(value);

        if (type.equals("paramfile"))
            return paramfileEditor(value);

        if (type.equals("keyword"))
            return keywordEditor(value);

        if (type.equals("coord"))
            return coordEditor(value, tooltp);

        // All other parameter types use a
        // text field as editor

        P2PPTextField textField = new P2PPTextField(value);
        editorComponent_ = textField;

        if (type.equals("integer"))
            textField.setDocument(intModel_);

        else if (type.equals("number"))
            textField.setDocument(floatModel_);

        else if (type.equals("numlist"))
            textField.setDocument(floatListModel_);

        else if (
            type.equals("intlist")
                || type.equals("pixel")
                || type.equals("intrect"))
            textField.setDocument(intListModel_);

        else if (type.equals("string"))
            textField.setDocument(new PlainDocument());

        textField.setText(value);
        textField.selectAll();
        textField.setToolTipText(tooltp_);
        return highlightComponent();
    }


    /** <br>ChangeLog: A M Chavan, 08-Aug-2001 */
    private Component booleanEditor(String value) {

        // The logic of this is pretty tricky: since the user clicked,
        // he wants to toggle the value of that cell. So we build a
        // new checkbox with an inverted value, and display that.

        if (value.equals("F"))
            editorComponent_ = new P2PPCheckBox(true);
        else
            editorComponent_ = new P2PPCheckBox(false);

        return highlightComponent();
    }


    /** <br>ChangeLog: A M Chavan, 08-Aug-2001 */
    private Component filenameEditor(String value) {

        p2ppFileChooser_ = new P2PPFileChooser(false);
        p2ppFileChooser_.showDialog(paramTable_, "Select File");
        editorComponent_ = new P2PPTextField((String)value);

        String filename = p2ppFileChooser_.getValue();
        JTextField jtf = (JTextField)editorComponent_; // syntactic sugar

        jtf.setDocument(new PlainDocument());

        if (filename != null && filename.length() > 0)
            jtf.setText(filename);
        else
            jtf.setText((String)value);

        jtf.setEditable(false);

        return highlightComponent();
    }


    /** <br>ChangeLog: A M Chavan, 08-Aug-2001 */
    private Component fileEditor(String value) {
        return fileEditor(value, false);
    }


    /** <br>ChangeLog: A M Chavan, 08-Aug-2001 */
    private Component paramfileEditor(String value) {
        return fileEditor(value, true);
    }


    /**
     * Deal with the editor of both "file" and "paramfile"
     * parameters -- they have a lot in common.
     *
     * @param value  Value to display: the actual file in the
     *               case of a "file" parameter; the filename
     *               in the case of a "paramfile" parameter.
     *
     * @param isParamfile True if the parameter is a "paramfile"
     *                    parameter; "false" otherwise.
     *
     * <br>ChangeLog: A M Chavan, 08-Aug-2001
     */
    private Component fileEditor(String value, boolean isParamfile) {

        // If <Shift-Button1> we just save the value of
        // the parameter to a file
        if (shiftPressed_) {
            savePFileParam(isParamfile, row_, column_, value);
            editorComponent_ = new P2PPTextField(value);
            return editorComponent_;
        }

        // If the file selection box is already
        // displayed we have really nothing to do
        if (filePreviewDialog_.isVisible())
            return editorComponent_;

        // Configure our file selection box
        // if the value is NODEFAULT, then we display nothing
        filePreviewDialog_.clearSelection();
        setFilePreviewExtension(currParam_.getRange());
        if (value.equals("NODEFAULT"))
            filePreviewDialog_.setValue("");
        else
            filePreviewDialog_.setValue(currParam_.getValue());

        if (parameterFileDirExists_ == false) {
            AppConfig appcfg = AppConfig.getAppConfig();
            String paramDir = appcfg.getParameterDir();
            JOptionPane.showMessageDialog(
                MainWindow.getPrincipalWindow(),
                "PARAMFILE.FOLDER '"
                    + paramDir
                    + "' not found.\n\nDefaulting to $HOME",
                "Warning message",
                JOptionPane.WARNING_MESSAGE);
            parameterFileDirExists_ = true;
        }

        filePreviewDialog_.setVisible(true); // Activate file selection box

        int returnValue = filePreviewDialog_.getReturnValue();
        if (returnValue != JOptionPane.OK_OPTION) {

            // User clicked on Cancel: we just cleanup and return
            editorComponent_ = new P2PPTextField(value);
            ((JTextField)editorComponent_).setEditable(false);
            return highlightComponent();
        }

        // User clicked on OK: see what we got

        File selected = filePreviewDialog_.getSelectedFileHandle();
        if (selected == null) { // IF   no selected file
            // User did nothing: we just cleanup and return
            editorComponent_ = new P2PPTextField(value);
            ((JTextField)editorComponent_).setEditable(false);
            return highlightComponent(); // THEN nothing to do
        }

        String pathname = selected.getAbsolutePath();
        String msg1 = "Error while attaching paramfile\n" + pathname + "\n";
        String msg2 = "\nParamfile was not attached.";

        if (!isParamfile) {

            // This is a 'file' parameter: check if it
            // is not binary, and if all goes well
            // copy the file text into the cell.

            String filetext = filePreviewDialog_.getValue();
            int asciiTestLength = 256;
            int i = TextUtils.isASCIIPrintable(filetext, asciiTestLength);
            if (i >= 0) {
                String msg =
                    msg1
                        + "The file contains non-ASCII characters (look at char "
                        + i
                        + ")"
                        + msg2;
                stdlog_.debug(msg);
                JOptionPane.showMessageDialog(
                    MainWindow.getPrincipalWindow(),
                    msg,
                    "Error in paramfile",
                    JOptionPane.ERROR_MESSAGE);
                return highlightComponent();
            }

            currParam_.setValue(filetext);
            editorComponent_ = new P2PPTextField(filetext);
            return highlightComponent();
        }

        // This is a 'paramfile' parameter: try to parse it and use whatever
        // information we find in it.
        Paramfile paramfile = null;
        try {
            paramfile = new Paramfile(pathname);
        } catch (NotASCIIException e) {
            String msg =
                msg1
                    + "The file contains non-ASCII characters (look "
                    + e.getMessage()
                    + ")"
                    + msg2;
            stdlog_.debug(msg);
            JOptionPane.showMessageDialog(
                MainWindow.getPrincipalWindow(),
                msg,
                "Error in paramfile",
                JOptionPane.ERROR_MESSAGE);
            return highlightComponent();
        } catch (Exception e) {
            String msg = msg1 + "Exception: " + e.getMessage() + msg2;
            stdlog_.debug(msg);
            JOptionPane.showMessageDialog(
                MainWindow.getPrincipalWindow(),
                msg,
                "Error in paramfile",
                JOptionPane.ERROR_MESSAGE);
            return highlightComponent();
        }
        
        String paf_name = paramfile.getValue("PAF.NAME");

        // Reject paramfiles which do not have a valid "PAF.NAME" keyword
        if (paf_name.length() == 0 || paf_name == null) {
            String msg = msg1 + "\"PAF.NAME\" keyword not found." + msg2;
            stdlog_.debug(msg);
            JOptionPane.showMessageDialog(
                MainWindow.getPrincipalWindow(),
                msg,
                "Error in paramfile",
                JOptionPane.ERROR_MESSAGE);
            return highlightComponent();
        }
        TemplateModel model = (TemplateModel)paramTable_.getModel();
        String instCode = model.getOD().getInstrument();
        // Reject paramfiles which do not have a valid "PAF.CHCK.CHECKSUM" keyword
        String errMsg = null;
        Vector instruments = AppConfig.getAppConfig().getAllowNoCheckSum();
        try {
            String keywordChksum = "PAF.CHCK.CHECKSUM";
            String pafChksum = paramfile.getValue(keywordChksum);
            if (pafChksum == null ||
                pafChksum.length() == 0 ||
                pafChksum.equals("0000000000000000") || 
                instruments.contains(instCode))
            {
                stdlog_.debug("!!!!!!!!! Nor computing Checksum");
                // Commented out:
                // Don't reject param files that don't have a checksum set yet
                // until other software is updated
                // errMsg = "\"" + keywordChksum + "\" not set correctly.";
            } else {
                // Reject the param file if it has a bad checksum
                long cklongValue = new Long(pafChksum).longValue();
                boolean isOK = CRC32PAFFile.validateChecksum(pathname,cklongValue);                
                if (!isOK) {
                    errMsg = "File is corrupt.\n" +
                        "Please contact ESO User Support Group.";
                }
            }
        } catch (FileNotFoundException fnf) {
            errMsg = "Can't find file.";
        } catch (IOException io) {
            errMsg = "Error reading file.";
        }
        if (errMsg != null) {
            errMsg = msg1 + errMsg + msg2;
            stdlog_.debug(errMsg);
            JOptionPane.showMessageDialog(
                MainWindow.getPrincipalWindow(),
                errMsg,
                "Paramfile Error",
                JOptionPane.ERROR_MESSAGE);
            return highlightComponent();
        }

        // Paramfile was valid, let's attach it to the parameter
        // -----------------------------------------------------

        ParamFileParameter pfParam = (ParamFileParameter)currParam_;
        pfParam.setValue(filePreviewDialog_.getValue());
        pfParam.setFilename(paf_name);
        editorComponent_ = new P2PPTextField(paf_name);

        // If this is an acquisition template,
        // initialize the Target from the paramfile
        //----------------------------------------

        // TODO Must review this: processing of paramfiles
        // TODO should take place at the OB level

        TemplateSignature ts = (TemplateSignature)model.getDataItem(column_);

        if (ts.getTemplateType().equals("acquisition")) {

            AcquisitionModel am = (AcquisitionModel)model;
            Target target = am.getOB().getTarget();
            try {
                target.updateFromParamfile(paramfile);
            } catch (Exception e) {
                String msg =
                    msg1
                        + e.getMessage()
                        + "\nTarget RA and/or Dec was not updated.";
                stdlog_.debug(msg);
                JOptionPane.showMessageDialog(
                    MainWindow.getPrincipalWindow(),
                    msg,
                    "Error in paramfile",
                    JOptionPane.ERROR_MESSAGE);
            }
            try {
                ConstraintSet cs = am.getOB().getConstraintSet();
                if (cs != null) {
                    cs.updateFromParamfile(paramfile);
                }
            } catch (Exception e) {
                String msg =
                    "Error while refreshing paramfile "
                        + paf_name
                        + ":\n"
                        + e.getMessage()
                        + "\nConstraint Set info was not updated.";
                JOptionPane.showMessageDialog(
                    MainWindow.getPrincipalWindow(),
                    msg,
                    "Error in paramfile",
                    JOptionPane.ERROR_MESSAGE);
            }

        }

        return highlightComponent();
    }


    /** <br>ChangeLog: A M Chavan, 09-Aug-2001 */
    private Component keywordEditor(String value) {

        // Vector comboEmtries = new Vector();
        boolean special = false;
        KeywordParameter key = (KeywordParameter)currParam_;
        String[] keylist = key.getKeywordList();
        P2PPComboBox combo = new P2PPComboBox(keylist);

        comboData_ = new Hashtable();
        editorComponent_ = combo;

        for (int i = 0; i < keylist.length; i++)
            if (keylist[i].equals("Special"))
                special = true;
            else
                comboData_.put(keylist[i], keylist[i]);

        combo.removeItem("Special"); // useless?
        if (special) {
            combo.setEditable(true);
            combo.setEditor(new P2PPComboBoxEditor());
        }

        String cp_val = currParam_.getValue();
        if (cp_val == null)
            cp_val = currParam_.getDefaultValue();

        if (comboData_.contains(cp_val)) {
            combo.setSelectedItem(cp_val);
        }

        return highlightComponent();
    }


    /** <br>ChangeLog: A M Chavan, 09-Aug-2001 */
    private Component coordEditor(String value, String tooltp) {

        CoordParameter coord = (CoordParameter)currParam_;
        CoordTextField coordTF = new CoordTextField(value);
        editorComponent_ = coordTF;

        if (coord.getRange().equals("ra"))
            coordTF.setDocument(rightAscensionModel_);
        else
            coordTF.setDocument(declinationModel_);

        if (value.equals(""))
            value = "00:00:00.000";

        coordTF.setText(value);
        coordTF.selectAll();
        coordTF.setToolTipText(tooltp);

        return highlightComponent();
    }


    /**
     * Set the border and associate a keyboard listener to
     * the current component.
     *
     * @return editorComponent_
     * <br>ChangeLog: A M Chavan, 08-Aug-2001; creation.
     */
    private Component highlightComponent() {

        JComponent jc = (JComponent)editorComponent_; // syntactic sugar
        jc.setBorder(BorderFactory.createLineBorder(Color.red, 1));
        editorComponent_.addKeyListener(keyListener_);

        return editorComponent_;
    }


    private void savePFileParam(
        boolean isParamfile,
        int row,
        int column,
        String value) {
        paramTable_.setRowSelectionInterval(row, row);
        paramTable_.setColumnSelectionInterval(column, column);
        // Java1.4 port fileSaveDialog_.clearSelection();
        fileSaveDialog_.cancelSelection();

        if (!value.equals("NODEFAULT")) {
            if (isParamfile) {

                // If this is a "paramfile" parameter,
                // initialize the file chooser with it
                File paramFile = null;
                paramFile =
                    new File(
                        fileSaveDialog_.getCurrentDirectory()
                            + File.separator
                            + value);
                fileSaveDialog_.setSelectedFile(paramFile);
            }
            int response = fileSaveDialog_.showSaveDialog(null);
            if (response == JFileChooser.APPROVE_OPTION) {
                saveParamFile(
                    currParam_.getValue(),
                    fileSaveDialog_.getSelectedFile());
            }
        }
        shiftPressed_ = false;
    }


    /**
     * Build the 'keywordlist' parameter grid component.
     *
     * @param value  Value of cell
     * @param window Top level ancestor (containing window)
     *
     * <br>ChangeLog:  A M Chavan, 24-Jan-01, SPR 1235
     */
    private Component keywordlistEditor(Object value, Frame window) {

        editorComponent_ = new P2PPTextField((String)value);
        KeywordParameter key = (KeywordParameter)currParam_;
        String[] keylist = key.getKeywordList();
        KeywordListParameter tKLP = (KeywordListParameter)key;
        KeywordListDialog tKLD = null;

        if (keywordListDialog_ != null && keywordListDialog_.isVisible()) {
            tKLD = keywordListDialog_;
            tKLD.initDestinationList(tKLP.getKeywords());
            return editorComponent_;
        }

        keywordListDialog_ =
            new KeywordListDialog(
                window,
                "Select from list",
                keylist,
                tKLP.getKeywords());

        keywordListDialog_.setLocationRelativeTo(window);

        keywordListDialog_.show();
        paramTable_.setRowSelectionInterval(row_, row_);
        ((P2PPTextField)editorComponent_).setText(keywordListDialog_.getValue());
        return editorComponent_;

        // editorComponent_ = new P2PPTextField();
        // JTextField tJTF = (JTextField)editorComponent_;
        //tJTF.setDocument( new PlainDocument() );
        //tJTF.setText( tKLD.getValue() );
        //tJTF.setEditable( false );
        //tJTF.setToolTipText( tooltp_ );
    }


    /**
     *  Inner class to bring up a key selection dialog
     */
    private class KeywordListDialog extends JDialog implements P2PPComponent {

        /** JList for source items */
        private JList srcList_;

        /** JList for destination items */
        private JList destList_;

        /** Source model for source list control */
        private DefaultListModel srcModel_;

        /** Destination model for destination list control */
        private DefaultListModel destModel_;
        //         /** Textfield for keyword date entry */
        //         private JTextField itemText_;

        /** Ok button for commiting selection */
        private JButton ok_;

        /** Cancel button for canceling dialog box */
        private JButton cancel_;

        /** Button that addItem to destination list */
        private JButton addItem_;

        /** Button that removed item from destination list */
        private JButton removeItem_;

        /** Panel to add controls to */
        private JPanel panel_;
        //         /** Button that adds keyword to source list */
        //         private JButton addKeyword_;

        /**
         * We store here a copy of the source values before editing
         * begins; we then use it to restore the original source
         * values if Cancel is clicked.
         *
         * <br>ChangeLog: A M Chavan, 24-Jan-01, SPR 1235.
         */
        private Object[] originalValues_;

        /** Layout manager for panel in dialog */
        GridBagConstraints c;


        /**
         * Create a KeywordListDialog
         * @param f Frame for dialog
         * @param title Title of dialog
         * @param srcValues source values for model in source JList
         * @param destValues dest source values for model in destination JList
         */
        KeywordListDialog(
            Frame f,
            String title,
            Object[] srcValues,
            Object[] destValues) {
            super(f, title, true);

            panel_ = new JPanel();
            panel_.setLayout(new GridBagLayout());
            c = new GridBagConstraints();
            c.weightx = 100;
            c.weighty = 100;
            c.gridx = GridBagConstraints.RELATIVE;
            c.gridy = 0;
            c.gridwidth = 1;
            c.gridheight = 5;
            c.fill = GridBagConstraints.BOTH;
            initLists(srcValues, destValues);
            srcList_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
            destList_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
            c.gridheight = 1;
            c.gridwidth = 0;
            c.weightx = 0;
            c.weighty = 0;
            c.fill = GridBagConstraints.NONE;
            c.anchor = GridBagConstraints.NORTH;
            c.gridy = GridBagConstraints.RELATIVE;
            ok_ = new JButton("    OK     ");
            cancel_ = new JButton("   Cancel  ");
            addItem_ = new JButton("    Add    ");
            removeItem_ = new JButton("   Delete  ");
            // addKeyword_  = new JButton("Add Keyword");
            panel_.add(ok_, c);
            panel_.add(cancel_, c);
            panel_.add(addItem_, c);
            panel_.add(removeItem_, c);
            // panel_.add(addKeyword_,c);
            c.fill = GridBagConstraints.BOTH;
            c.gridy = 6;
            c.gridx = 0;
            c.gridwidth = 5;
            c.weightx = 100;
            c.weighty = 100;
            c.anchor = GridBagConstraints.SOUTH;
            // itemLabel_ = new JLabel("Add keyword ");
            // panel_.add(itemLabel_,c);
            // c.gridx++;
            // itemText_ = new JTextField();
            // panel_.add(itemText_,c);
            setContentPane(panel_);
            addActionListeners();
            pack();

            if (f != null) {
                setLocation(f.getLocation());
            }
        }


        /**
         * Initialise the source and destination lists
         * @param srcValues used to creare model for source list
         * @param destValues used to ceate model for destination list
         */
        private void initLists(Object[] srcValues, Object[] destValues) {

            srcModel_ = new DefaultListModel();
            if (srcValues != null) {
                for (int i = 0; i < srcValues.length; i++)
                    srcModel_.addElement(srcValues[i]);
            }
            srcList_ = new JList(srcModel_);

            initDestinationList(destValues);

            final JScrollPane srcScrollPane =
                new JScrollPane(
                    srcList_,
                    ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            panel_.add(srcScrollPane, c);

            final JScrollPane destScrollPane =
                new JScrollPane(
                    destList_,
                    ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            panel_.add(destScrollPane, c);
        }


        /**
         * Set value to be displated in destination list (currently
         * selected keywords). Here we also set the originalValues_
         * field, will be used by the Cancel action.
         *
         * @param destValues List of values to be displayed in the
         *                   destination list
         *
         * <br>ChangeLog: A M Chavan, 24-Jan-01, SPR 1235.
         */
        public void initDestinationList(Object[] destValues) {

            destModel_ = new DefaultListModel();

            if (destValues != null) {

                originalValues_ = new Object[destValues.length];
                for (int i = 0; i < destValues.length; i++) {

                    destModel_.addElement(destValues[i]);
                    originalValues_[i] = destValues[i];
                }
            }

            destList_ = new JList(destModel_);
        }


        /**
         * Add action listeners for this dialog
         */
        private void addActionListeners() {
            addItem_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    if (srcList_.getSelectedValue() != null) {
                        int srcIndex = srcList_.getSelectedIndex();
                        destModel_.addElement(srcModel_.elementAt(srcIndex));
                    }
                }
            });

            removeItem_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    if (destList_.getSelectedValue() != null) {
                        destModel_.removeElementAt(destList_.getSelectedIndex());
                    }
                }
            });

            ok_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    currParam_.setValue(getValue());
                    setVisible(false);
                    dispose();
                }
            });

            cancel_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {

                    /* Cancel behaves same as OK, but
                       first restores the values we initially
                       stored in originalValues_.
                       A M Chavan, 24-Jan-01, SPR 1235
                    */

                    initDestinationList(originalValues_);
                    currParam_.setValue(getValue());
                    setVisible(false);
                    dispose();
                }
            });
        }


        /**
         * Return value to be displated in grid
         * @return value to be displayed
         */
        public String getValue() {
            StringBuffer value = new StringBuffer();
            for (int i = 0; i < destModel_.getSize(); i++) {
                String item = (String)destModel_.getElementAt(i);
                value.append(item);
                if (i < destModel_.getSize() - 1)
                    value.append(" ");
            }
            return value.toString();
        }
    }


    /**
     * This class allows a user to select a file as a parameter.
     */
    private class P2PPFileChooser
        extends SwingFileChooser implements P2PPComponent {

        /** Name of current file selected */
        private String currentFile_;


        /**
         * Create a file selection control
         * @param boolean multiSelectionEnabled
         */
        P2PPFileChooser(boolean multiSelectionEnabled) {
            super();
            setMultiSelectionEnabled(multiSelectionEnabled);
            setFileSelectionMode(FILES_ONLY);
        }


        /** Set the file name */
        public void setValue(String value) {
            currentFile_ = value;
        }


        /** Return a file handle fron the file selection control */
        public String getValue() {
            String value = "";
            if (!isMultiSelectionEnabled()) {
                File f = getSelectedFile();
                if (f != null)
                    value = f.getAbsolutePath();
                else
                    value = currentFile_;
            }
            return value;
        }
    }


    /**
     * This class allows a user to Preview a file in the FileChooser Dialog
     */
    private class P2PPFilePreviewDialog
        extends FilePreviewDialog implements P2PPComponent {

        P2PPFilePreviewDialog(Frame f, String title) {
            super(f, title);
            if (f != null) {
                setLocation(f.getLocation());
            }
        }


        public String getValue() {
            //if(getReturnValue() == JOptionPane.OK_OPTION) {
            return super.getValue();
            // }
        }


        public javax.swing.filechooser.FileFilter getAcceptAllFileFilter() {
            return chooser.getAcceptAllFileFilter();
        }


        public void P2PPsetWorkingDirectory(File directory) {
            chooser.setCurrentDirectory(directory);
        }
    }


    /**
     * Inner clas to render a checkbox in the grid
     */
    private class P2PPCheckBox extends JCheckBox implements P2PPComponent {

        /**
         * Create a  P2PPCheckBox
         * @param value value text beside checkbox
         * @param state set the checkbox on
         */
        P2PPCheckBox(boolean state) {
            super(null, null, state);
            //setValue(value);
            //setBorderPainted(true);
            // TODO Set the background to white!!!
            setFocusTraversalKeys(
                KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.UP_CYCLE_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
        }


        /**
         * Retrun a refernce to the multi-line tool tip
         * @return a multi line JToolTip
         */
        public JToolTip createToolTip() {
            return multiLineToolTip_;
        }


        /**
         * Set the value of the checkbox accoding to the string value
         * @param value
         */
        public void setValue(String value) {
            if (value.equals("T")) {
                setSelected(true);
                //setText("True");
            } else {
                setSelected(false);
                //setText("False");
            }
        }


        /**
         * Return value to be displated in grid
         * @return value to be displayed
         */
        public String getValue() {
            return isSelected() ? "T" : "F";
        }
    }


    /**
     *  Inner clas to render a combobox in the grid
     *  This class is an editable comboBox
     */
    public static class P2PPComboBox extends JComboBox implements P2PPComponent {

        /** Append values to the ComboBox */
        private boolean append_;

        /** append String buffer */
        private StringBuffer appendStr_;


        /**
         * Construct a comboBox with a list of objects
         */
        P2PPComboBox(Object[] items) {
            super(items);
            append_ = false;
            appendStr_ = new StringBuffer();
            setFocusTraversalKeys(
                KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.UP_CYCLE_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
        }


        public JToolTip createToolTip() {
            return multiLineToolTip_;
        }


        public void setAppend(boolean append) {
            append_ = append;
        }


        /**
         * Return value to be displayed in grid
         * @return value selected
         */
        public String getValue() {
            return (String)getSelectedItem();
        }
    }


    private void initKeyListener() {
        keyListener_ = new KeyListener() {
            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_TAB
                    || KeyEvent.getKeyText(evt.getKeyCode()).equals("Shift-Tab")
                    || evt.getKeyCode() == KeyEvent.VK_ENTER
                    || evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    paramTable_.stopCellEditing();
                    paramTable_.processKeyEvent(evt);
                }
                //stdlog_.debug("Key Pressed " + evt);
            }
            public void keyReleased(KeyEvent evt) {
            }
            public void keyTyped(KeyEvent evt) {
            }
        };
    }


    /**
     *  Inner clas to render a textfield in the grid
     */
    public class P2PPTextField extends JTextField implements P2PPComponent {

        P2PPTextField(String value) {
            super(value);
            if (value.equals("NODEFAULT"))
                setText("");
            else {
                setText((String)value);
            }
        }


        public JToolTip createToolTip() {
            return multiLineToolTip_;
        }


        /**
         * Construct the textfield with default text value
         */
        P2PPTextField(String value, InputMaskModel model) {
            super(value);
            setDocument(model);
            if (value.equals("NODEFAULT"))
                setText("");
            else {
                setText((String)value);
            }
            setFocusTraversalKeys(
                KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
            setFocusTraversalKeys(
                KeyboardFocusManager.UP_CYCLE_TRAVERSAL_KEYS,
                Collections.EMPTY_SET);
        }


        /** Default Constructor */
        P2PPTextField() {
            super();
        }


        /**
         * Return value to be displayed in grid
         * @return value entered
         */
        public String getValue() {
            String text = getText();
            if (text.length() == 0) {
                return currParam_.getDefaultValue();
            }
            return (String)getText();
        }
    }


    public class CoordTextField extends P2PPTextField implements P2PPComponent {

        /** constructor */
        CoordTextField(String value) {
            super(value);
            setText((String)value);
        }


        /**
         * Construct the textfield with default text value
         */
        CoordTextField(String value, InputMaskModel model) {
            super(value);
            setDocument(model);
            setText((String)value);
        }


        /** Default Constructor */
        CoordTextField() {
            super();
        }


        /**
         * Create tootip for component
         * @return JToolTip
         */
        public JToolTip createToolTip() {
            return multiLineToolTip_;
        }


        /**
         * Return value to be displayed in grid
         * @return value entered
         */
        public String getValue() {
            String text = getText();
            if (text != null && text.length() > 0) {
                if (currParam_.verifyValue(text)) {
                    DegreeAdaptor oa = null;
                    if (currParam_.getRange().equalsIgnoreCase("dec"))
                        oa = (DegreeAdaptor)AdaptorFactory.getAdaptor(
                            Target.class, "Declination");
                    else
                        oa = (DegreeAdaptor)AdaptorFactory.getAdaptor(
                             Target.class, "RA");
                    Object obj = oa.toMetaLevelObject(text);
                    text = oa.fromMetaLevelObject(obj);
                }
            }
            if (text == null || text.length() == 0)
                return currParam_.getDefaultValue();
            return text;
        }
    }


    public class ComboKeyListener implements KeyListener {
        public void keyPressed(KeyEvent evt) {
            if (evt.getKeyCode() == KeyEvent.VK_TAB
                || KeyEvent.getKeyText(evt.getKeyCode()).equals("Shift-Tab")
                || evt.getKeyCode() == KeyEvent.VK_ENTER
                || evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                paramTable_.processKeyEvent(evt);
            }
        }


        public void keyReleased(KeyEvent evt) {
            return;
        }


        public void keyTyped(KeyEvent evt) {
            return;
        }
    }


    public class P2PPComboBoxEditor extends BasicComboBoxEditor {

        public P2PPComboBoxEditor() {
            editor = new P2PPTextField("");
            editor.addFocusListener(this);
            editor.addKeyListener(comboKeyList_);
            editor.setBorder(null);
        }
    }
}

