package org.eso.ohs.phase2.icdVcs;

import org.apache.log4j.Logger;
import org.eso.vlt.base.Ccs.CcsException;

/**
 * Listener for events from the BOB interface
 * 
 * <p>
 * NOTE: This class must be public in order for Java/Ccs to work.<BR>
 *       I'm not sure of the reason but it's probably to do with
 *       their use of the Reflection API.
 * 
 * @version $Revision: 1.7 $
 * @author Bob Kemp
 */
public class EventCmds implements EventCmdListener {

    private static final String rcsid =
        "$Id: EventCmds.java,v 1.7 2004/04/08 12:09:53 tcanavan Exp $";

    /**
     * <br>ChangeLog: amchavan, 05-Apr-2002: added
     */
    private static Logger stdlog_ = Logger.getLogger(EventCmds.class);

    // @author amchavan, 05-Apr-2002: commented out
    // log window for displaying messages received from the BOB interface 
    // private Log log_;

    /**
     * ScriptCmds object that passes BOs to BOB.
     * Used to get references to the last CB/OB that was sent,
     * so the status can be updated.
     */
    private ScriptCmds scriptLnr_;

    /**
     * Default constructor.
     * @exception CcsException CCS throws an exception
     */
    public EventCmds(ScriptCmds scriptLnr) throws CcsException {
        scriptLnr_ = scriptLnr;
    }

    /**
      * Handler method for ObsBlockStatus messages from BOB.
      * 
      * @param msg	    status message as a tcl array
      * @return	    null string
      */
    public void ObsBlockStatus(
        String id,
        String when,
        String what,
        String extraInfo) {

        stdlog_.info(when + " " + id + ": " + what + " " + extraInfo);

        scriptLnr_.setBusObjStatusFromEventType(what, id);
    }

    /**
     * Handler method for TemplateStatus messages from BOB.
     * 
     * @param msg	    status message as a tcl array
     * @return	    null string
     */
    public void TemplateStatus(
        String id,
        String when,
        String what,
        String extraInfo) {
        stdlog_.info(when + " " + id + ": " + what + " " + extraInfo);
        scriptLnr_.setTemplateStatusFromEventType(extraInfo,id,when);
    }

    /**
     * Handler method for GetPAF messages from BOB.
     * 
     * @param reply	    Reply as a string, eg Mask Manufacturing Report.
     */
    public void GetPAFResult(String reply) {
        // do nothing -- intended for MaskTracker software.
    }

}
