package org.eso.ohs.phase2.orang;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.SchedRun;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.dfs.RunProperties;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;

/**
 * 
 * This class loads OB information using bulding OBTargetConstraint objects, and
 * its associated TimeIntervals
 * 
 * @author pnunes
 * 
 * Created on Aug 3, 2005
 */
public class OBInfoLoader {

    /*
     * Constant for the stardard time interval in the TimeInterval object
     */
    private static final String STANDARD_TIME_INTERVAL = "T";

    /**
     * Logger
     */
    protected static Logger stdlog_ = Logger.getLogger(OBInfoLoader.class);

    /*
     * Hashtable containing the indexes of each OB in the obData_ array key:
     * Long ob_id value: Integer index
     */
    private Hashtable obIndexes_ = new Hashtable();

    /*
     * Hashtable containing the indexes in the obData_ array, associated with
     * each run key: Long run_id value: Vector indexes (Long objects)
     */
    private Hashtable runIndexes_ = new Hashtable();

    /**
     * @param obIds
     * @return
     * @throws ObjectNotFoundException
     * @throws ObjectIOException
     */
    public ObDataContainer[] load(long[] obIds)
            throws ObjectNotFoundException, ObjectIOException {

        // // select the OBs using an in clause, will clearly not perform for a
        // big
        // // amount of records
        //
        // UnpackedStorableObject[] obList =
        // OpcDbaseMgr.getInstance().read(null,
        // Phase2SelectStmt.wcObTargetConstraintInfoList(obIds),
        // ObTargetConstraintInfo.class);
        // stdlog_.debug(" _____________________- " + obList.length);
        //
        // // retrieve the time intervals
        // UnpackedStorableObject[] timeIntList =
        // OpcDbaseMgr.getInstance().read(
        // Phase2SelectStmt.wcObsBlocksTable(),
        // Phase2SelectStmt.wcTimeConstraintInfoList(obIds),
        // TimeInterval.class);
        //
        // // retrieve the sched_rep from OPC
        // UnpackedStorableObject[] schedRepList = OpcDbaseMgr
        // .getInstance()
        // .read(Phase2SelectStmt.wcObsBlocksTable(),
        // Phase2SelectStmt.wcSchedRepObsBlocksList(obIds),
        // SchedRun.class);
        //
        //
        // // retrieve the usg_file from OPC
        // UnpackedStorableObject[] usgFileList =
        // OpcDbaseMgr.getInstance().read(
        // obsBlocksAndOtherTables,
        // Phase2SelectStmt.wcUsgFileObsBlocksList(obsBlocksTableWhere),
        // UsgFile.class);
        //
        //        
        // createObDataContainers(obList, timeIntList, schedRepList);

        ObDataContainer[] obData = new ObDataContainer[0];
        return obData;
    }

    /**
     * Filters out the TOO obs (the one having DEC and Ra = 0) and builds the
     * associated mapping. The mapping will be used to add related information
     * (run properties, rank class,...) to each ob.
     * 
     * @param obList
     * @return ObTargetConstraintInfo[]
     */
    protected ObDataContainer[] buildObArray(UnpackedStorableObject[] obList) {
        ObTargetConstraintInfo obInfo = null;

        // initialise indexes and ObContainer array
        Vector obDataVector = new Vector();

        int cuurentObIndex = 0;
        for (int i = 0; i < obList.length; i++) {

            obInfo = (ObTargetConstraintInfo) obList[i].getObject();
            if (!isToo(obInfo)) {
                // store the OB in the container
                obDataVector.add(new ObDataContainer(obInfo));

                // build ob index hashtable
                Integer currentIndexObj = new Integer(cuurentObIndex);
                obIndexes_.put(new Long(obInfo.getDbaseId()), currentIndexObj);

                // build run vector
                Vector vec;
                if (runIndexes_.containsKey(new Long(obInfo.getRunId()))) {
                    vec = (Vector) runIndexes_.get(new Long(obInfo.getRunId()));
                } else {
                    vec = new Vector();
                }

                vec.add(currentIndexObj);
                cuurentObIndex++;
                runIndexes_.put(new Long(obInfo.getRunId()), vec);
            }

        }
        return (ObDataContainer[]) obDataVector
                .toArray(new ObDataContainer[obDataVector.size()]);
    }

    /**
     * Verify if an ob is a TOO type.
     * 
     * @param obInfo
     *            ob to be analysed
     * @return boolean
     */
    private boolean isToo(ObTargetConstraintInfo obInfo) {
        boolean result = false;
        if (obInfo != null) {
            if (obInfo.getDec() == 0 && obInfo.getRa() == 0) {
                result = true;
            }
        }
        return result;
    }

    // load all OBs given a where condition (in the obs_blocks table) ,
    // joining with time intervals
    public ObDataContainer[] load(String otherTables, String obsBlocksTableWhere)
            throws ObjectNotFoundException, ObjectIOException {

        UnpackedStorableObject[] obList = OpcDbaseMgr.getInstance().read(
                otherTables, obsBlocksTableWhere, ObTargetConstraintInfo.class);

        if (otherTables == null)
            otherTables = "";

        // join condition for time intervals
        String intervalObBlocksWhere = Phase2SelectStmt
                .includeJoinTimeIntervalObBlocks(obsBlocksTableWhere);
        String obsBlocksAndOtherTables = Phase2SelectStmt.wcObsBlocksTable()
                + otherTables;

        // retrieve the time intervals
        UnpackedStorableObject[] timeIntList = OpcDbaseMgr.getInstance().read(
                obsBlocksAndOtherTables, intervalObBlocksWhere,
                TimeInterval.class);

        // retrieve the sched_rep from OPC
        UnpackedStorableObject[] schedRepList = OpcDbaseMgr.getInstance().read(
                obsBlocksAndOtherTables,
                Phase2SelectStmt.wcSchedRepObsBlocksList(obsBlocksTableWhere),
                SchedRun.class);

        // retrieve the usg_file from OPC
        UnpackedStorableObject[] runPropList = OpcDbaseMgr.getInstance().read(
                obsBlocksAndOtherTables,
                Phase2SelectStmt.wcRunPropObsBlocksList(obsBlocksTableWhere),
                RunProperties.class);

        ObDataContainer[] obData = createObDataContainers(obList, timeIntList,
                schedRepList, runPropList);

        return obData;
    }

    /**
     * @param obList
     * @param timeIntList
     * @param schedRepList
     * @param usgFileList
     * @return
     */
    protected ObDataContainer[] createObDataContainers(
            UnpackedStorableObject[] obList,
            UnpackedStorableObject[] timeIntList,
            UnpackedStorableObject[] schedRepList,
            UnpackedStorableObject[] usgFileList) {
        ObDataContainer[] obData = buildObArray(obList);
        obData = appendTimeIntervalsToOB(timeIntList, obData);
        obData = appendSchedRunToOB(schedRepList, obData);
        obData = appendRunPropToOB(usgFileList, obData);
        return obData;
    }

    //  

    /**
     * load all OBs from a queue
     * 
     * @param b
     * @param string
     * 
     * @param queueId
     * @return an array of OBDataContainer
     * @throws ObjectNotFoundException
     * @throws ObjectIOException
     */
    public ObDataContainer[] loadQueueOBs(String queueName, String user,
            boolean isSuperUser)
            throws ObjectNotFoundException, ObjectIOException {

        String[] queues = new String[1];
        queues[0] = queueName;

        String obsBlocksTableWhere = Phase2SelectStmt.includeJoinSchedule(
                queues, user, isSuperUser);
        String schedTables = Phase2SelectStmt.wcScheduleTables("");

        ObDataContainer[] obContainerList = load(schedTables,
                obsBlocksTableWhere);

        return obContainerList;
    }

    public ObDataContainer[] loadQueues(String[] queueNames, String user,
            boolean isSuperUser)
            throws ObjectNotFoundException, ObjectIOException {

        String obsBlocksTableWhere = Phase2SelectStmt.includeJoinSchedule(
                queueNames, user, isSuperUser);
        String schedTables = Phase2SelectStmt.wcScheduleTables("");

        ObDataContainer[] obContainerList = load(schedTables,
                obsBlocksTableWhere);

        return obContainerList;

    }

    // load all OBs from a telescope given a where condition, joining with
    // queues and time intervals
    public ObDataContainer[] loadTelescopeOBs(String telescope, String user,
            boolean isSuperUser)
            throws ObjectNotFoundException, ObjectIOException {

        String obsBlocksTableWhere = Phase2SelectStmt
                .includeJoinScheduleTelescope(telescope, user, isSuperUser);
        String schedTables = Phase2SelectStmt.wcScheduleTables("");

        ObDataContainer[] obContainerList = load(schedTables,
                obsBlocksTableWhere);

        return obContainerList;
    }

    /**
     * @param timeIntList
     * @return TimeInterval[]
     */
    private ObDataContainer[] appendTimeIntervalsToOB(
            UnpackedStorableObject[] timeIntList, ObDataContainer[] obData) {

        TimeInterval[] timeInt = new TimeInterval[timeIntList.length];

        for (int i = 0; i < timeInt.length; i++) {
            timeInt[i] = (TimeInterval) timeIntList[i].getObject();

            // associate this time interval with its corresponding
            // ObDataContainer
            long obId = timeInt[i].getObId();
            Integer obIdRef = (Integer) obIndexes_.get(new Long(obId));
            if (obIdRef != null) {
                int index = obIdRef.intValue();

                // check time interval type
                if (timeInt[i].getType().equals(STANDARD_TIME_INTERVAL))
                    obData[index].addTimeInterval(timeInt[i]);
                else
                    obData[index].addSiderealTimeInterval(timeInt[i]);
            }
        }
        return obData;
    }

    /**
     * @param schedRunList
     */
    private ObDataContainer[] appendSchedRunToOB(
            UnpackedStorableObject[] schedRunList, ObDataContainer[] obData) {

        SchedRun schedRun = null;

        for (int i = 0; i < schedRunList.length; i++) {
            schedRun = (SchedRun) schedRunList[i].getObject();

            // associate this sched run with its corresponding ObDataContainers
            long runId = schedRun.getRunId();

            if (runIndexes_.containsKey(new Long(runId))) {
                Vector vec = (Vector) runIndexes_.get(new Long(runId));

                for (Iterator iter = vec.iterator(); iter.hasNext();) {
                    Integer index = (Integer) iter.next();
                    obData[index.intValue()].setSchedRun(schedRun);
                }

            }

        }
        return obData;
    }

    /**
     * @param usgFileList
     * @param obData
     */
    private ObDataContainer[] appendRunPropToOB(
            UnpackedStorableObject[] usgFileList, ObDataContainer[] obData) {

        RunProperties usgFile = null;

        for (int i = 0; i < usgFileList.length; i++) {
            usgFile = (RunProperties) usgFileList[i].getObject();

            // associate this sched run with its corresponding ObDataContainers
            long runId = usgFile.getDbaseId();

            if (runIndexes_.containsKey(new Long(runId))) {
                Vector vec = (Vector) runIndexes_.get(new Long(runId));

                for (Iterator iter = vec.iterator(); iter.hasNext();) {
                    Integer index = (Integer) iter.next();
                    obData[index.intValue()].setRunProperties(usgFile);
                }

            }

        }
        return obData;
    }

}